/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package kernel;

import java.io.BufferedWriter;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import kernel.functions.FuncFunction;
import kernel.functions.FunctionsFactory;
import kernel.functions._Function;
import simulation.SimulationSpec;
import simulation.components.ImageComponent;
import simulation.objects.AiryPoint;
import ui.simulation.frames.NoisesDialog;

public class Generator
{
	
	public static final int GENERATE_FROM_BEGINING			= 1;
	public static final int GENERATE_FROM_DISTRIBUTION		= 2;
	public static final int GENERATE_FROM_GENERATION		= 3;
	public static final int GENERATE_FROM_NOISES			= 4;
	public static final int GENERATE_FROM_NORMALIZATION		= 5;
	

	
	
	public static final void generate(final SimulationSpec simulSpec, final int iGenerationType)
	{
		try{
			/* on va lancer la generation dans un autre thread */
			new Thread(){
				public void run(){
					
					/* passe en mode generation */
					simulSpec.setSimulating(true);
					simulSpec.fireLog("");
					simulSpec.fireLog("::*******************");
					simulSpec.fireLog("::STARTING GENERATION");
					simulSpec.fireLog("::*******************");
					simulSpec.fireLog(" | ");
					simulSpec.fireLog(" | ");
					
					switch(iGenerationType){
		
						case GENERATE_FROM_BEGINING:
							/* distribution des objets */
							generateFromObjects(simulSpec);
							break;
							
						case GENERATE_FROM_DISTRIBUTION:
							/* distribution des objets */
							generateFromObjects(simulSpec);
							break;
		
						case GENERATE_FROM_GENERATION:
							/* genere les objets */
							generateFromGeneration(simulSpec);
							break;
							
						case GENERATE_FROM_NOISES:
							/* genere les objets */
							generateFromNoises(simulSpec);
							break;
							
						case GENERATE_FROM_NORMALIZATION:
							/* genere les objets */
							generateFromNormalization(simulSpec);
							break;
		
					}
					
					/* fin de la simulation */
					simulSpec.fireLog(" | ");
					simulSpec.fireLog(" \\_  fin.");
					simulSpec.setSimulating(false);
		

				}
			}.start();
		}catch(Exception ex) {Kernel.fireError(Generator.class.getName(), "generateAll()", ex);}
	}


	
	
	
	
	
	
	private static final void generateFromObjects(final SimulationSpec simulSpec)
	{
		try{
			simulSpec.fireSimulationStartDistribution();
			simulSpec.fireLog(" | :: DISTRIBUTION D'OBJETS ::");
			simulSpec.fireLog(" |  | ");
			
			/* vide la liste actuelle */
			simulSpec.clearComponents();
			/* ajoute les nouveaux composants */
			String[] tabl = cleanText(simulSpec.DISTRIBUTION_SCRIPT).split("\n");

			int nb = 0;
			for(int i=0; i<tabl.length; i++){
				try{
					if(tabl[i].startsWith("[")){ // [ x_range ; y_range ; flux_range ; nb ]
						nb += distributePoint(simulSpec, tabl[i]);
					}else{
						if((tabl[i] != null) && (tabl[i].length() > 0)){
							/* fonction de distribution */
							nb += distributeFunction(simulSpec, tabl[i]);
						}
					}
				}catch(Exception ex) {Kernel.fireError(Generator.class.getName(), "distributeObjects()", ex);}
			}
			
			/* logs */
			simulSpec.fireLog(" |  | total : " + nb + " objet(s) distribués");

			/* notifie les changements */
			simulSpec.fireSimulationComponentsListChanged();
			
			
			
			/* regarde ce qu'il faut faire */
			if( simulSpec.GENERATION_AUTOMATIC == true ){
				/* passe a la generation */
				simulSpec.fireLog(" |  | ");
				simulSpec.fireLog(" |  \\_fin.");
				simulSpec.fireLog(" | ");
				generateFromGeneration(simulSpec);
			}else{
				/* fin temporaire de la simulation */
				simulSpec.getMatrix().resetMatrix();
				simulSpec.fireLog(" |  | ");
				simulSpec.fireLog(" |  \\_fin.");
				simulSpec.fireLog(" | ");
			}

		}catch(Exception ex) {Kernel.fireError(Generator.class.getName(), "generateFromObjects()", ex);}
	}
	
	
	


	private static final void generateFromGeneration(final SimulationSpec simulSpec)
	{
		try{

			simulSpec.fireSimulationStartGeneration();
			simulSpec.fireLog(" | :: GENERATION DES OBJETS ::");

			/* on initialise la matrice */
			simulSpec.getMatrix().resetMatrix();

			/* on génère les composants */
			Vector lstComponents = simulSpec.getComponents();
			if(lstComponents != null){
				int j = 0;
				/* on genere les elements */
				for(int i=0; i<lstComponents.size(); i++){
					((ImageComponent)(lstComponents.get(i))).generate();
					j++;
					/* met a jour que tout les 100 objets */
					if(j > 100) {j = 0; simulSpec.fireSimulationGeneratedComponentsChanged(i);}
				}
			}
			
			simulSpec.fireLog(" |  | ");
			simulSpec.fireLog(" |  \\_fin.");
			simulSpec.fireLog(" | ");
			
			/* regarde ce qu'il faut faire */
			if( simulSpec.NOISIFICATION_AUTOMATIC == true ) generateFromNoises(simulSpec);

		}catch(Exception ex) {Kernel.fireError(Generator.class.getName(), "generateFromGeneration()", ex);}
	}


	
	
	
	private static final void generateFromNoises(final SimulationSpec simulSpec)
	{
		try{					
			FuncFunction func = null;
			
			/* passe en mode generation */
			simulSpec.fireSimulationStartNoisation();
			simulSpec.fireLog(" | :: GENERATION DES BRUITS ::");
			simulSpec.fireLog(" |  | ");

			/* parse l'expression */
			String strFormule = computeFonction(simulSpec.NOISES_FORMULA);
			
			/* remplaces les variables globales */
			strFormule = Kernel.replace(strFormule, NoisesDialog.VAR_OBS_TIME, ""+simulSpec.PARAM_OBSERVATION_TIME);
			strFormule = Kernel.replace(strFormule, NoisesDialog.VAR_PIX_RES, ""+simulSpec.PARAM_PIXEL_RESOLUTION);
			strFormule = Kernel.replace(strFormule, NoisesDialog.VAR_TELESC_DIAM, ""+simulSpec.PARAM_TELESCOPE_DIAMETER);
			strFormule = Kernel.replace(strFormule, NoisesDialog.VAR_LAMBDA, ""+simulSpec.getFilterMono().lambdaCenterInMeter());
			strFormule = Kernel.replace(strFormule, NoisesDialog.VAR_DLAMBDA, ""+simulSpec.getFilterMono().dlambdaInMeter());

			strFormule = Kernel.replace(strFormule, NoisesDialog.VAR_OBJECTS_ELECT, FuncFunction.VAR_OBJECTS_ELECT);
			
			System.out.println(strFormule);
			//Maths.fromJyPerSteradToElecPerPixel(func.yValue(spec.getFilterMono().lambdaCenterInMeter()), spec.getFilterMono(), spec.getObservationTime(), spec.getTelescDiameter(), spec.getPixelField());

			
			/* tranforme en formule */
			func = (FuncFunction)(FunctionsFactory.parse(null, null, "func:"+strFormule, null));

			/* on regarde s'il y a du bruit */
			if(func == null){
				simulSpec.fireLog(" |  | Impossible de parser la fonction, aucun bruit.");
				for(int i=0; i<simulSpec.PARAM_IMAGE_SIZE_X; i++){
					for(int j=0; j<simulSpec.PARAM_IMAGE_SIZE_Y; j++){
						simulSpec.getMatrix().setNoisesElectrons(i, j, 0.0);
					}
				}
			}else{
				simulSpec.fireLog(" |  | Fonction correctement parsee, ajout du bruit...");
				/* on applique le bruit pour chaque pixel */
				for(int i=0; i<simulSpec.PARAM_IMAGE_SIZE_X; i++){
					for(int j=0; j<simulSpec.PARAM_IMAGE_SIZE_Y; j++){
						func.setVariable(FuncFunction.VAR_OBJECTS_ELECT, simulSpec.getMatrix().getObjectsElectrons(i, j));
						simulSpec.getMatrix().setNoisesElectrons(i, j, func.yValue(0.0));
					}
				}
			}
			
			simulSpec.fireLog(" |  | ");
			simulSpec.fireLog(" |  \\_fin.");
			simulSpec.fireLog(" | ");
			
			/* on passe a la suite */
			generateFromNormalization(simulSpec);

		}catch(Exception ex) {Kernel.fireError(Generator.class.getName(), "generateFromNoises()", ex);}
	}
	
	
	
	
	

	
	private static final void generateFromNormalization(final SimulationSpec spec)
	{
		try{
			
			/* passe en mode generation */
			spec.fireSimulationStartNormalization();
			spec.fireLog(" | :: NORMALISATION ET CREATION DE L'IMAGE ::");
			
			/* on normalise */
			spec.getMatrix().normalize();

			/* on cree l'image */
			spec.getMatrix().createImage();
			
			spec.fireLog(" |  | ");
			spec.fireLog(" |  \\_fin.");
			spec.fireLog(" | ");
			
		}catch(Exception ex) {Kernel.fireError(Generator.class.getName(), "generateFromNormalization()", ex);}
	}
	
	
	

	
	
	
	
	
	
	
	
	
	
	
	
	
	
	

	
	
	private static final String computeFonction(final String strSource)
	{
		String strDest = null;
		try{
			if(strSource == null) return null;
			/* texte a etudier */
			String[] tabl = cleanText(strSource).split("\n");
			if(tabl == null) return null;
			/* variables locales */
			String currentLine;
			String currentName;
			String currentValue;
			Object key = null;
			Hashtable hash = new Hashtable();
			/* on parcours les lignes de la derniere a la premiere */
			for(int i=tabl.length-1; i>=0; i--){
				/* retrouve la ligne courante */
				currentLine = tabl[i];
				/* verifie que c'est bien une variable */
				if(currentLine.indexOf(":") > 0){
					/* nom de cette variable */
					currentName = currentLine.substring(0, currentLine.indexOf(":"));
					/* formule courante */
					currentValue = "(" + currentLine.substring(currentLine.indexOf(":")+1) + ")";
					/* on remplace les sous fonctions */
					for(Enumeration e = hash.keys(); e.hasMoreElements();){
						key = e.nextElement();
						if(key != null) currentValue = Kernel.replace(currentValue, "%"+((String)(key))+"%", (String)(hash.get(key)));
					}
					/* on regarde si c'est la fonction principale */
					if("RETURN".equalsIgnoreCase(currentName) == true){
						/* fonction principale */
						strDest = currentValue;
					}else{
						/* on memorise la fonction */
						hash.put(currentName, currentValue);
					}
				}
			}
		}catch(Exception ex) {Kernel.fireError(Generator.class.getName(), "computeFonction()", ex);}
		return strDest;
	}
	
	
	
	
	private static final String cleanText(final String strSource)
	{
		try{
			if(strSource == null) return "";
			/* gere les diferents retours a la ligne */
			String strDest = "";
			String strMod = Kernel.replace(strSource, "\r", "\n");
			strMod = Kernel.replace(strMod, "\n\n", "\n");
			/* parcours la chaine */
			boolean blComm = false;
			for(int i=0; i<strMod.length(); i++){
				/* on regarde si on est dans un commentaire */
				if(blComm == true){
					/* on regarde si une fin de ligne est trouvee */
					if(strMod.charAt(i) == '\n'){
						blComm = false;
						strDest = strDest + strMod.charAt(i);
					}
				}else{
					if(strMod.charAt(i) == '#'){
						/* on regarde si un commentaire commance */
						blComm = true;
					}else{
						/* on regarde si on doit garder le caractere */
						if((strMod.charAt(i) != '\t') && (strMod.charAt(i) != ' ')){
							strDest = strDest + strMod.charAt(i);
						}
					}
				}
			}
			return strDest;
		}catch(Exception ex) {Kernel.fireError(Generator.class.getName(), "cleanText()", ex);}
		return strSource;
	}
	
	
	
	
	
	private static final void distributeObject(final SimulationSpec spec, final int iNumber, final double fluxMin, final double fluxMax, final double xMin, final double xMax, final double yMin, final double yMax)
	{
		try{
			double X, Y, flux;
			/* nombre fixe */
			for(int i=0; i<iNumber; i++){
				/* position x */
				if(xMin == xMax) X = xMin;
				else X = Maths.randomDouble(xMin, xMax);
				/* position y */
				if(yMin == yMax) Y = yMin;
				else Y = Maths.randomDouble(yMin, yMax);
				/* flux */
				if(fluxMin == fluxMax) flux = fluxMin;
				else flux = Maths.randomDouble(fluxMin, fluxMax);
				/* ajoute */
				spec.addComponent(new AiryPoint(spec, X, Y, flux));
			}
		}catch(Exception ex) {Kernel.fireError(Generator.class.getName(), "distributeObject()", ex);}
	}

	private static final int distributePoint(final SimulationSpec spec, final String strPoint)
	{
		try{
			double fluxMin, fluxMax, xMin, xMax, yMin, yMax;
			String[] tablPoint;
			String[] tablPonc;
			int j;
			int nb = 0;
			/* objets ponctuels */
			String str = Kernel.replace(strPoint.toUpperCase(), "XMAX", ""+(spec.PARAM_IMAGE_SIZE_X-1));
			str = Kernel.replace(str, "YMAX", ""+(spec.PARAM_IMAGE_SIZE_Y-1));
			
			tablPonc = ("]"+str+"[").split("\\]\\[");
			for(j=0; j<tablPonc.length; j++){
				try{
					if((tablPonc[j] != null) && (tablPonc[j].length() > 3)){
						tablPoint = tablPonc[j].split("\\;");
						/* position x */
						if(tablPoint[0].indexOf("..") > -1){
							xMin = Double.parseDouble(tablPoint[0].split("\\.\\.")[0]);	
							xMax = Double.parseDouble(tablPoint[0].split("\\.\\.")[1]);
						}else{
							xMin = Double.parseDouble(tablPoint[0]);
							xMax = xMin;
						}
						/* position y */
						if(tablPoint[1].indexOf("..") > -1){
							yMin = Double.parseDouble(tablPoint[1].split("\\.\\.")[0]);
							yMax = Double.parseDouble(tablPoint[1].split("\\.\\.")[1]);
						}else{
							yMin = Double.parseDouble(tablPoint[1]);
							yMax = yMin;
						}
						/* flux */
						if(tablPoint[3].indexOf("..") > -1){
							fluxMin = Double.parseDouble(tablPoint[3].split("\\.\\.")[0]);
							fluxMax = Double.parseDouble(tablPoint[3].split("\\.\\.")[1]);
						}else{
							fluxMin = Double.parseDouble(tablPoint[3]);
							fluxMax = fluxMin;
						}
						/* ajoute */
						distributeObject(spec, Integer.parseInt(tablPoint[2]), fluxMin, fluxMax, xMin, xMax, yMin, yMax);
						nb += Integer.parseInt(tablPoint[2]);
					}
				}catch(Exception ex) {Kernel.fireError(Generator.class.getName(), "distributePoint()*", ex);}
			}
			return nb;
		}catch(Exception ex) {Kernel.fireError(Generator.class.getName(), "distributePoint()", ex);}
		return 0;
	}

	
	
	private static final int distributeFunction(final SimulationSpec spec, final String strFunction)
	{
		try{
			
			_Function func = null;
			
			/* recupere la fonction de distribution */
			for(int i=0; i<spec.DISTRIBUTION_FUNCTIONS.size(); i++){
				if( strFunction.equalsIgnoreCase( ((_Function)(spec.DISTRIBUTION_FUNCTIONS.get(i))).getName() ) ) func = ((_Function)(spec.DISTRIBUTION_FUNCTIONS.get(i)));
			}
			
			
			if(func == null){
				spec.fireLog(" |  | erreur, fonction inconnue : " + strFunction);
				return 0;
			}else{
				spec.fireLog(" |  | distribution de : " + strFunction);
			}
			

			
			/* surface en steradian */
			final double imgSurface = Maths.fromArcsecSqToSteradian(spec.PARAM_PIXEL_RESOLUTION * spec.PARAM_PIXEL_RESOLUTION) * spec.PARAM_IMAGE_SIZE_X * spec.PARAM_IMAGE_SIZE_Y;
			
			final double step = 0.1;
			double realX;
			double numberPerSterad;
			double numberPerImage;
			int roundNumberPerImage;

			int nb = 0;
			
			for(double x=-8.0; x<4.0; x=x+step){
				
				/* valeur reelle e x, c'est a dire le flux en Jy */
				realX = Maths.pow( 10.0 , x + ( step / 2.0 ) ) ;
				/* valeur en un point */
				numberPerSterad = func.yValue( realX );
				
				/* on regarde si la valeur est definie */
				if(numberPerSterad != Maths.NaN){
					//spec.fireLog(" |  | ste : 10^" + x + " = " + numberPerSterad);
					numberPerImage = numberPerSterad * imgSurface; // nombre dans notre image
					//spec.fireLog(" |  | img : 10^" + x + " = " + numberPerImage);
					roundNumberPerImage = (int) (numberPerImage + 0.49); // nombre arrondi
					//spec.fireLog(" |  | rnd : 10^" + x + " = " + roundNumberPerImage);
					distributeObject(spec, roundNumberPerImage, realX, realX, 0, spec.PARAM_IMAGE_SIZE_X, 0, spec.PARAM_IMAGE_SIZE_Y);
					nb += roundNumberPerImage;
				}
			}
			spec.fireLog(" |  |  | " + nb + " objet(s) distribués");
			spec.fireLog(" |  |  \\_ok.");
			spec.fireLog(" |  | ");
			return nb;
		}catch(Exception ex) {Kernel.fireError(Generator.class.getName(), "distributeFunction()", ex);}
		spec.fireLog(" |  |  \\_erreur.");
		return 0;
	}
	

	
	


	
	
	
	
	
	
	
	
	

	
	
	
	
	
	
	
	
	
	
	
	

	
	
	
	private static final String getInfo(final String strTitle, final String strValue)
	{
		String str = " " + strTitle + " ";
		try{
			for(int i=0; i<(58-strTitle.length()-strValue.length()); i++) str += ".";
			str = str + " " + strValue + "\n";
		}catch(Exception ex) {Kernel.fireError(Generator.class.getName(), "getInfo()", ex);}
		return str;
	}



	public static final String generateInfos(final SimulationSpec spec, final boolean blAllData, final BufferedWriter w)
	{
		String str1 = "", str2 = "";
		try{
			str1= "-------------------------------------------------------------\n"
                + "                    Simulation results                       \n"
                + "-------------------------------------------------------------\n"
                + " Copyright (C) 2006 SOS Project <http://sos.project.free.fr> \n"
                + " Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>      \n"
                + " Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>      \n"
                + "-------------------------------------------------------------\n"
				+ "                                                             \n"
				+ "GENERAL INFORMATIONS                                         \n"
				+ getInfo("pixel resolution", spec.PARAM_PIXEL_RESOLUTION + " arcsec")
				+ getInfo("image size", spec.PARAM_IMAGE_SIZE_X + " x " + spec.PARAM_IMAGE_SIZE_Y + " px")
				+ getInfo("image resolution", (spec.PARAM_IMAGE_SIZE_X*spec.PARAM_PIXEL_RESOLUTION) + " x " + (spec.PARAM_IMAGE_SIZE_Y*spec.PARAM_PIXEL_RESOLUTION) + " arcsec")
				+ getInfo("filter", (spec.getFilterMono().lambdaCenterInMeter()*1e6) + " µm, delta : " + (spec.getFilterMono().dlambdaInMeter()*1e6) + " µm")
				+ getInfo("filter desc.", spec.getFilterMono().getDescription())
				+ "                                                             \n"
				+ "TECHNICAL ASPECTS                                            \n"
				+ getInfo("pause time", spec.PARAM_OBSERVATION_TIME + " sec")
				+ getInfo("telesc. diameter", spec.PARAM_TELESCOPE_DIAMETER + " m")
				+ "                                                             \n"
				+ "IMAGE PROPERTIES                                             \n"
				+ getInfo("normalization delta", ""+spec.NORMALISATION_VALUE)
				+ getInfo("normalization power", ""+spec.NORMALISATION_POWER)
				+ getInfo("generation limit", ""+spec.GENERATION_OBJECTS_SIZE + " px")
				+ "                                                             \n"
				+ "COMPONENTS                                                   \n"
				;
			
			
			if(spec.getComponents().size() > 0){
				ImageComponent c;
				str1 += getInfo("count", ""+spec.getComponents().size());
				
				if(w != null) w.write(str1);
				
				int i;
				for(i=0; (i<spec.getComponents().size())&&(blAllData||((!blAllData)&&(i<1000))); i++){
					c = (ImageComponent)(spec.getComponents().get(i));
					if(w == null){
						str1 += getInfo("airy point #"+(i+1), Maths.toString(c.getRealTotalFluxMONO(), 3) + " Jy ["+Maths.toString(c.getRealTotalElectNbMONO(), 2) + " e-] ("+Maths.toString(c.getPosX(), 1)+","+Maths.toString(c.getPosY(), 1)+")");	
					}else{
						w.write(getInfo("airy point #"+(i+1), Maths.toString(c.getRealTotalFluxMONO(), 3) + " Jy ["+Maths.toString(c.getRealTotalElectNbMONO(), 2) + " e-] ("+Maths.toString(c.getPosX(), 1)+","+Maths.toString(c.getPosY(), 1)+")"));
					}
				}
				if((!blAllData) && (i >= 1000)){
					str1+= " and more... (export as text file for complete list)         \n";
				}
			}else{

				str1+= " no component.                                               \n";
				
				if(w != null) w.write(str1);
			}
			str2+= "                                                             \n"
				+ "                                                             \n"
			;
			
			if(w != null) w.write(str2);
			
		}catch(Exception ex) {Kernel.fireError(Generator.class.getName(), "generateInfos()", ex);}
		return str1 + str2;
	}

	
	
	
	
	
	

	

}
