/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package kernel;

import java.awt.Color;
import java.awt.Component;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;

import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import simulation.SimulationSpec;

public class Kernel
{

	public static final String APPLET_VERSION			= "1.006a";
	public static final String APPLET_TITLE				= "sos.project simulator v." + APPLET_VERSION;
	
	public static int APPLET_WIDTH						= 540;
	public static int APPLET_HEIGHT						= 420;
	public static Color APPLET_COLOR					= new Color(255, 255, 255);
	
	public static boolean DEBUG_SIMUL					= true;
	
	private static JTextArea txtLogs					= null;


	

	
	
	
	
	
	
	
	
	
	

	public static final String replace(final String strSource, final String strToFind, final String strToPut)
	{
		try{
			if((strSource == null) || (strToFind == null)) return "";
			boolean blFind;
			int iTxtLen = strSource.length();
			int iFindLen = strToFind.length();
			int i = 0;
			String strDone = "";
			String strPut = "";
			if(strToPut != null) strPut = strToPut;
			while(i < iTxtLen){
				blFind = false;
				if(i <= (iTxtLen-iFindLen)){
					if(strToFind.equals(strSource.substring(i, i+iFindLen))){
						/* on vient de trouver un texte a remplacer */
						strDone = strDone + strPut;
						i = i + iFindLen;
						blFind = true;
					}
				}
				if(blFind == false){
					/* on ajoute ce caractere et on passe au suivant */
					strDone = strDone + strSource.charAt(i);
					i = i + 1;
				}
			}
			return strDone;
		}catch(Exception ex) {fireError(Kernel.class.getName(), "replace()", ex);}
		return strSource;
	}

	
	
	

	
	
	
	public static final void saveImage(final BufferedImage img, final String strFormat, final Component parent)
	{
		try{
			
			if(img == null) return;
			
			File file = askForSaveFile(parent);
		 	if(file != null){
		 		if(file.getAbsolutePath().toLowerCase().endsWith("." + strFormat) == false) file = new File(file.getAbsolutePath() + "." + strFormat);
		 		ImageIO.write(img, strFormat, file);
		 	}
		}catch(Exception ex) {fireError(Kernel.class.getName(), "saveImage()", ex);}
	}
	
	public static final void saveSimulation(final SimulationSpec spec, final Component parent)
	{
		try{
			File file = askForSaveFile(parent);
		 	if(file != null){
		 		if(file.getAbsolutePath().toLowerCase().endsWith(".txt") == false) file = new File(file.getAbsolutePath() + ".txt");
		 		BufferedWriter w = new BufferedWriter(new FileWriter(file));
		 		Generator.generateInfos(spec, true, w);
		 		w.flush();
		 		w.close();
		 	}
		}catch(Exception ex) {fireError(Kernel.class.getName(), "saveSimulation()", ex);}
	}
	
	
	
	

	
	
	
	
	public static final File askForSaveFile(final Component parent)
	{
		try{
			JFileChooser chooser = new JFileChooser();
			chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
			chooser.setDialogTitle("Fichier de destination...");
			chooser.setSelectedFile(null);
		    int retval = chooser.showSaveDialog(parent);
		    if(retval == JFileChooser.APPROVE_OPTION){
		    	 File theFile = chooser.getSelectedFile();
		    	 if(theFile != null) return theFile;
		    }
		}catch(Exception ex){
			if(ex instanceof java.security.AccessControlException){
				JOptionPane.showMessageDialog(parent ,"Impossible de parcourir les fichiers locaux depuis une Applet.\nPour cela, vous devez lancer l'application en local.", "Parcourir les fichiers", JOptionPane.OK_OPTION);
			}else{
				JOptionPane.showMessageDialog(parent ,"Impossible de parcourir les fichiers locaux.\nSi ce n'est pas le cas, lancez l'application en local.", "Parcourir les fichiers", JOptionPane.OK_OPTION);
				fireError(Kernel.class.getName(), "askForSaveFile()", ex);
			}
		}
		return null;
	}

	
	
	
	
	
	
	public static final void setLogger(final JTextArea txt) {txtLogs = txt;}
	
	public static final void fireError(String strClass, String strFunc, Exception e)
	{
		try{
			
			String strMessage = "** [ERROR] ** " + strClass + "_" + strFunc + " : " + e.getClass().getName();
			System.out.println(strMessage);
			e.printStackTrace();

			if(txtLogs != null){
				if((txtLogs.getText() == null) || (txtLogs.getText().length() == 0)) txtLogs.setText(strMessage);
				else txtLogs.setText(txtLogs.getText() + "\n" + strMessage);
				SwingUtilities.invokeLater(
					new Runnable(){
						public void run(){
							try{
								txtLogs.updateUI();
								txtLogs.scrollRectToVisible(new Rectangle(1, txtLogs.getHeight()+100, 1, 1));
							}catch(Exception ex) {}
						}
					}
				);
			}
		}catch(Exception ex) {}
	}
	

}
