/*
 SOS Project Classes.

 Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
 SOS Project is :
 Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
 Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package kernel;

public class Maths
{

	public static final double	h	= 6.6260693e-34;			// kg.m[+2].s[-1]
																// == J.s

	public static final double	c	= 299792458;				// m.s[-1]

	public static final double	k	= 1.380650524e-23;			// kg.m[+2].s[-2].K[-1]

	public static final double	pi	= Math.PI;

	public static final double	hc	= h * c;

	public static final double	NaN	= Double.NEGATIVE_INFINITY;

	// Hz = s[-1]
	// Jy = 10[-26].W.m[-2].Hz[-1] = 10[-26].kg.s[-2]
	// W = kg.m[+2].s[-3]
	//
	//

	public static final String toString(final double a, int iNumber)
	{
		String str = "";
		String puiss = "";
		String test = ("" + a).toLowerCase();
		if (iNumber == -1) return test;
		if (iNumber > 0) {
			if (test.indexOf("e") > -1) {
				str = ""
						+ (long) Math.floor(((Double.parseDouble(test
								.substring(0, test.indexOf("e"))) * Math.pow(
								10, iNumber)) + 0.49));
				puiss = test.substring(test.indexOf("e"));
			} else {
				str = ""
						+ (long) Math
								.floor(((a * Math.pow(10, iNumber)) + 0.49));
			}
			/* on ajoute le bon nombre de 0 */
			while (str.length() < (iNumber + 1))
				str = "0" + str;
			/* on recherche la position du point */
			int point = str.length() - iNumber;
			/* on ajoute le point */
			str = str.substring(0, point) + "." + str.substring(point);
			/* on supprime les zero a la fin */
			while (str.endsWith("0"))
				str = str.substring(0, str.length() - 1);
			/* on supprime le point s'il est a la fin */
			if (str.endsWith(".")) str = str.substring(0, str.length() - 1);
		}
		return str + puiss;
	}

	public static final double randomDouble(final double dblMin,
			final double dblMax)
	{
		return ((Math.random() * (dblMax - dblMin)) + dblMin);
	}

	public static final int randomInt(final int iMin, final int iMax)
	{
		return (int) ((Math.random() * (iMax - iMin)) + iMin);
	}

	public static int randomPoisson(double c)
	{

		int x = 0;
		double t = 0.0;

		for (;;) {

			t = t - Math.log(Math.random()) / c;
			if (t > 1.0)

			break;
			x = x + 1;
		}
		return x;
	}

	// public static long ramdomGaussMC = 0;

	public static double randomGaussMC(double ro)
	{
		double rnd = 0.0;
		double max = Maths.gauss(0, 0, ro);
		// long i = 0;
		do {
			rnd = Maths.randomDouble(-4.0 * ro, +4.0 * ro);
			// i++;
		} while (Maths.gauss((rnd < 0 ? -rnd : rnd), 0.0, ro) < Maths
				.randomDouble(0, max));
		// if(i > ramdomGaussMC){
		// ramdomGaussMC = i;
		// System.out.println(i);
		// }
		return rnd;
	}

	public static final double fromKpcToArcsec(double R, double D)
	{
		return R / D / 4.84813681109536e-6;
	} // return R / D / pi * 180 * 3600

	public static final double fromArcsecToKpc(double R, double D)
	{
		return R * D * 4.84813681109536e-6;
	} // return R * D * pi / 180 / 3600

	public static final double fromDegreeToRadian(double teta)
	{
		return teta * 0.017453292519943295;
	} // return teta / 180.0 * pi

	public static final double fromRadianToDegree(double teta)
	{
		return teta * 57.29577951308232;
	} // return teta * 180.0 / pi

	public static final double fromArcsecToRadian(double teta)
	{
		return teta * 4.84813681109536e-6;
	} // return teta * pi / 3600 / 180

	public static final double fromRadianToArcsec(double teta)
	{
		return teta * 206264.80624709636;
	} // return teta / pi * 3600 * 180

	public static final double fromArcsecSqToSteradian(double teta)
	{
		return teta * 2.3504430539097885e-11;
	}

	public static final double fromSteradianToArcsecSq(double teta)
	{
		return teta * 4.254517029615221e10;
	}

	public static final double fromJyToElec(double flux_Jy, double lambda_M,
			double dlambda_M, double t_Sec, double diamTel_M)
	{
		double Atel = pi * diamTel_M * diamTel_M / 4.0;
		double dnu = (c * dlambda_M) / (lambda_M * lambda_M);
		return flux_Jy * 1.0e-26 * t_Sec * dnu * Atel * lambda_M / hc;
	}

	public static final double fromElecToJy(double flux_Elec, double lambda_M,
			double dlambda_M, double t_Sec, double diamTel_M)
	{
		double Atel = pi * diamTel_M * diamTel_M / 4.0;
		double dnu = (c * lambda_M) / (lambda_M * lambda_M);
		return flux_Elec / 1.0e-26 / t_Sec / dnu / Atel / lambda_M * hc;
	}

	public static final double fromJyPerSteradToElecPerPixel(
			double flux_JyPerSterad, double lambda_M, double dlambda_M,
			double t_Sec, double diamTel_M, double pixRes_Arcsec)
	{
		return fromJyToElec(flux_JyPerSterad, lambda_M, dlambda_M, t_Sec,
				diamTel_M)
				* fromArcsecSqToSteradian(pixRes_Arcsec * pixRes_Arcsec);
	}

	public static final double fromElecPerPixelToJyPerSterad(
			double flux_ElecPerPixel, double lambda_M, double dlambda_M,
			double t_Sec, double diamTel_M, double pixRes_Arcsec)
	{
		return fromElecToJy(flux_ElecPerPixel, lambda_M, dlambda_M, t_Sec,
				diamTel_M)
				/ fromArcsecSqToSteradian(pixRes_Arcsec * pixRes_Arcsec);
	}

	/**
	 * Calcule la distance entre deux point.
	 * 
	 * @param x1 Position X du point 1.
	 * @param y1 Position Y du point 1.
	 * @param x2 Position X du point 2.
	 * @param y2 Position Y du point 2.
	 * 
	 * @return La distance euclidienne entre le point 1 et le point 2, ou un
	 *         nombre négatif en cas d'erreur.
	 */
	public static final double euclidianDistance(final double x1,
			final double y1, final double x2, final double y2)
	{
		try {
			/* calcul classique d'une distance entre deux points */
			return Math.sqrt((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));
		} catch (Exception ex) {
		}
		return 0.0;
	}

	/**
	 * Calcule le rayon en fonction de x et y dans une géométrie axisymétrique.
	 * 
	 * @param x
	 * @param y
	 * @param PAx
	 * @param Qx
	 * @return
	 */
	public static final double axisymRay(final double x, final double y,
			final double PAx, final double Qx)
	{
		try {
			return Math.sqrt(Math.pow(x * Math.cos(PAx) + y * Math.sin(PAx),
					2.0)
					/ Math.pow(Qx, 2.0)
					+ Math.pow(y * Math.cos(PAx) - x * Math.sin(PAx), 2.0));
		} catch (Exception ex) {
		}
		return 0.0;
	}

	public static final double log(final double a)
	{
		if (a == NaN) return a;
		return Math.log(a) / Math.log(10);
	}

	public static final double ln(final double a)
	{
		return Math.log(a);
	}

	public static final double pow(final double a, final double b)
	{
		return Math.pow(a, b);
	}

	public static final double planck(final double lambda, final double T)
	{
		try {
			double v = c / lambda;
			return (2.0 * h * Math.pow(v, 3.0))
					/ (Math.pow(c, 2.0) * (Math.exp(((h * v) / (k * T))) - 1.0));
		} catch (Exception ex) {
		}
		return NaN;
	}

	public static final double planck2(final double lambda, final double T)
	{
		try {
			double c1 = 3.7418310e-16; // 2 × π × h × c^2
			double c2 = 1.43879e-2; // h × c / k
			return (c1 * Math.pow(lambda, -5.0))
					/ (pi * Math.exp(c2 / (lambda * T)) - 1.0);
		} catch (Exception ex) {
		}
		return NaN;
	}

	public static double gauss(double x, double mu, double ro)
	{
		try {
			return Math.exp(-0.5 * Math.pow((x - mu) / ro, 2.0))
					/ (ro * Math.sqrt(2.0 * Maths.pi));
		} catch (Exception ex) {
		}
		return NaN;
	}

	public static double poisson(int x, double lambda)
	{
		try {
			return Math.exp(-lambda) * Math.pow(lambda, x) / factorial(x);
		} catch (Exception ex) {
		}
		return NaN;
	}

	/**
	 * Calcule et retourne la fonction de Bessel, d'ordre 0.
	 * 
	 * @param x Valeur de x.
	 * @return La fonction de Bessel, d'ordre 0.
	 */
	public static final double j0(double x)
	{
		try {
			double ax;
			if ((ax = Math.abs(x)) < 8.0) {
				double y = x * x;
				double ans1 = 57568490574.0
						+ y
						* (-13362590354.0 + y
								* (651619640.7 + y
										* (-11214424.18 + y
												* (77392.33017 + y
														* (-184.9052456)))));
				double ans2 = 57568490411.0
						+ y
						* (1029532985.0 + y
								* (9494680.718 + y
										* (59272.64853 + y
												* (267.8532712 + y * 1.0))));
				return ans1 / ans2;
			} else {
				double z = 8.0 / ax;
				double y = z * z;
				double xx = ax - 0.785398164;
				double ans1 = 1.0
						+ y
						* (-0.1098628627e-2 + y
								* (0.2734510407e-4 + y
										* (-0.2073370639e-5 + y * 0.2093887211e-6)));
				double ans2 = -0.1562499995e-1
						+ y
						* (0.1430488765e-3 + y
								* (-0.6911147651e-5 + y
										* (0.7621095161e-6 - y * 0.934935152e-7)));
				return Math.sqrt(0.636619772 / ax)
						* (Math.cos(xx) * ans1 - z * Math.sin(xx) * ans2);
			}
		} catch (Exception ex) {
		}
		return NaN;
	}

	/**
	 * Calcule et retourne la fonction de Bessel, d'ordre 1.
	 * 
	 * @param x Valeur de x.
	 * @return La fonction de Bessel, d'ordre 1.
	 */
	static public final double j1(double x)
	{
		try {
			double ax;
			double y;
			double ans1, ans2;
			if ((ax = Math.abs(x)) < 8.0) {
				y = x * x;
				ans1 = x
						* (72362614232.0 + y
								* (-7895059235.0 + y
										* (242396853.1 + y
												* (-2972611.439 + y
														* (15704.48260 + y
																* (-30.16036606))))));
				ans2 = 144725228442.0
						+ y
						* (2300535178.0 + y
								* (18583304.74 + y
										* (99447.43394 + y
												* (376.9991397 + y * 1.0))));
				return ans1 / ans2;
			} else {
				double z = 8.0 / ax;
				double xx = ax - 2.356194491;
				y = z * z;
				ans1 = 1.0
						+ y
						* (0.183105e-2 + y
								* (-0.3516396496e-4 + y
										* (0.2457520174e-5 + y
												* (-0.240337019e-6))));
				ans2 = 0.04687499995
						+ y
						* (-0.2002690873e-3 + y
								* (0.8449199096e-5 + y
										* (-0.88228987e-6 + y * 0.105787412e-6)));
				double ans = Math.sqrt(0.636619772 / ax)
						* (Math.cos(xx) * ans1 - z * Math.sin(xx) * ans2);
				if (x < 0.0) ans = -ans;
				return ans;
			}
		} catch (Exception ex) {
		}
		return NaN;
	}

	/**
	 * Calcule et retourne la fonction de Bessel, d'ordre n.
	 * 
	 * @param n Ordre de la fonction.
	 * @param x Valeur de x.
	 * @return La fonction de Bessel, d'ordre n.
	 */
	public static final double jn(int n, double x)
	{
		try {
			int j, m;
			double ax, bj, bjm, bjp, sum, tox, ans;
			boolean jsum;
			double ACC = 40.0;
			double BIGNO = 1.0e+10;
			double BIGNI = 1.0e-10;
			if (n == 0) return j0(x);
			if (n == 1) return j1(x);
			ax = Math.abs(x);
			if (ax == 0.0) {
				return 0.0;
			} else {
				if (ax > ((double) n)) {
					tox = 2.0 / ax;
					bjm = j0(ax);
					bj = j1(ax);
					for (j = 1; j < n; j++) {
						bjp = j * tox * bj - bjm;
						bjm = bj;
						bj = bjp;
					}
					ans = bj;
				} else {
					tox = 2.0 / ax;
					m = 2 * ((n + (int) Math.sqrt(ACC * n)) / 2);
					jsum = false;
					bjp = ans = sum = 0.0;
					bj = 1.0;
					for (j = m; j > 0; j--) {
						bjm = j * tox * bj - bjp;
						bjp = bj;
						bj = bjm;
						if (Math.abs(bj) > BIGNO) {
							bj *= BIGNI;
							bjp *= BIGNI;
							ans *= BIGNI;
							sum *= BIGNI;
						}
						if (jsum) sum += bj;
						jsum = !jsum;
						if (j == n) ans = bjp;
					}
					sum = 2.0 * sum - bj;
					ans /= sum;
				}
			}
			return (((x < 0.0) && (n % 2 == 1)) ? -ans : ans);
		} catch (Exception ex) {
		}
		return NaN;
	}

	/**
	 * Calcule et retourne la fonction de Bessel de seconde sorte, d'ordre 0.
	 * 
	 * @param x Valeur de x.
	 * @return La fonction de Bessel de seconde sorte, d'ordre 0.
	 */
	public static final double y0(double x)
	{
		try {
			if (x < 8.0) {
				double y = x * x;
				double ans1 = -2957821389.0
						+ y
						* (7062834065.0 + y
								* (-512359803.6 + y
										* (10879881.29 + y
												* (-86327.92757 + y * 228.4622733))));
				double ans2 = 40076544269.0
						+ y
						* (745249964.8 + y
								* (7189466.438 + y
										* (47447.26470 + y
												* (226.1030244 + y * 1.0))));
				return (ans1 / ans2) + 0.636619772 * j0(x) * Math.log(x);
			} else {
				double z = 8.0 / x;
				double y = z * z;
				double xx = x - 0.785398164;
				double ans1 = 1.0
						+ y
						* (-0.1098628627e-2 + y
								* (0.2734510407e-4 + y
										* (-0.2073370639e-5 + y * 0.2093887211e-6)));
				double ans2 = -0.1562499995e-1
						+ y
						* (0.1430488765e-3 + y
								* (-0.6911147651e-5 + y
										* (0.7621095161e-6 + y
												* (-0.934945152e-7))));
				return Math.sqrt(0.636619772 / x)
						* (Math.sin(xx) * ans1 + z * Math.cos(xx) * ans2);
			}
		} catch (Exception ex) {
		}
		return NaN;
	}

	/**
	 * Calcule et retourne la fonction de Bessel de seconde sorte, d'ordre 1.
	 * 
	 * @param x Valeur de x.
	 * @return La fonction de Bessel de seconde sorte, d'ordre 1.
	 */
	public static final double y1(double x)
	{
		try {
			if (x < 8.0) {
				double y = x * x;
				double ans1 = x
						* (-0.4900604943e13 + y
								* (0.1275274390e13 + y
										* (-0.5153438139e11 + y
												* (0.7349264551e9 + y
														* (-0.4237922726e7 + y * 0.8511937935e4)))));
				double ans2 = 0.2499580570e14
						+ y
						* (0.4244419664e12 + y
								* (0.3733650367e10 + y
										* (0.2245904002e8 + y
												* (0.1020426050e6 + y
														* (0.3549632885e3 + y)))));
				return (ans1 / ans2) + 0.636619772
						* (j1(x) * Math.log(x) - 1.0 / x);
			} else {
				double z = 8.0 / x;
				double y = z * z;
				double xx = x - 2.356194491;
				double ans1 = 1.0
						+ y
						* (0.183105e-2 + y
								* (-0.3516396496e-4 + y
										* (0.2457520174e-5 + y
												* (-0.240337019e-6))));
				double ans2 = 0.04687499995
						+ y
						* (-0.2002690873e-3 + y
								* (0.8449199096e-5 + y
										* (-0.88228987e-6 + y * 0.105787412e-6)));
				return Math.sqrt(0.636619772 / x)
						* (Math.sin(xx) * ans1 + z * Math.cos(xx) * ans2);
			}
		} catch (Exception ex) {
		}
		return NaN;
	}

	/**
	 * Calcule et retourne la fonction de Bessel de seconde sorte, d'ordre n.
	 * 
	 * @param n Ordre de la fonction.
	 * @param x Valeur de x.
	 * @return La fonction de Bessel de seconde sorte, d'ordre n.
	 */
	public static final double yn(int n, double x)
	{
		try {
			double by, bym, byp, tox;
			if (n == 0) return y0(x);
			if (n == 1) return y1(x);
			tox = 2.0 / x;
			by = y1(x);
			bym = y0(x);
			for (int j = 1; j < n; j++) {
				byp = j * tox * by - bym;
				bym = by;
				by = byp;
			}
			return by;
		} catch (Exception ex) {
		}
		return NaN;
	}

	/**
	 * Calcule et retourne la valeur factorielle de n.
	 * 
	 * @param n Le nombre dont il faut calculer la valeur factorielle.
	 * @return La valeur factorielle de n.
	 */
	public static final double factorial(final int n)
	{
		try {
			if (n < 0) return 0;
			if (n < 2) return 1;
			long fact = 2;
			for (int i = 3; i <= n; i++)
				fact = fact * i;
			return fact;
		} catch (Exception ex) {
		}
		return NaN;
	}

}
