/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package kernel.functions;

import java.awt.Color;

import kernel.Kernel;
import kernel.Maths;
import kernel.jep_adds.FromJyPerSteradToElecPerPixel;
import kernel.jep_adds.Gauss;
import kernel.jep_adds.J1;
import kernel.jep_adds.Planck;
import kernel.jep_adds.RandomGaussMC;

import org.nfunk.jep.JEP;

public class FuncFunction implements _Function
{

	public static final String VAR_OBJECTS_ELECT			= "objects_elec";


	private String strName									= null;
	private String strDesc									= null;
	private String strFunc									= null;
	
	private Color color										= null;

	private JEP myParser;

	private boolean isReady									= false;


	
	
	
	
	

	public FuncFunction(final String strName, final String strDesc, final String strFunc, final Color color)
	{
		try{
			this.color = color;
			this.strName = strName;
			this.strDesc = strDesc;
			this.strFunc = strFunc;
			this.initParser(strFunc);
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "DistribFunction()", ex);}
	}
	
	

	
	
	

	private final void initParser(final String strFunc)
	{
		try{
			
			if((strFunc == null) || (strFunc.length() < FunctionsFactory.MATH_FUNCTION.length())) return;
			
			String strData = strFunc.substring(FunctionsFactory.MATH_FUNCTION.length());

			/* fonction */
			myParser = new JEP();
			myParser.setImplicitMul(true);
			/* ajoute les fonctions par default */
			myParser.addStandardFunctions();
			/* ajoute les fonctions locales */
			myParser.addFunction("j1", new J1());
			myParser.addFunction("planck", new Planck());
			myParser.addFunction("gauss", new Gauss());
			myParser.addFunction("gaussmcr", new RandomGaussMC());
			myParser.addFunction("fromJyPerSteradToElecPerPixel", new FromJyPerSteradToElecPerPixel());
			/* ajoute les constantes par default */
			myParser.addStandardConstants();
			/* ajoute les constantes locales */
			myParser.addConstant("k", new Double(Maths.k));
			myParser.addConstant("h", new Double(Maths.h));
			myParser.addConstant("c", new Double(Maths.c));
			myParser.addConstant("NaN", new Double(Maths.NaN));
			/* ajoute la variable principale */
			//myParser.addVariable("x", new Double(0.0));
			/* ajoute les autres variables */
			myParser.addVariable(VAR_OBJECTS_ELECT, new Double(0.0));
			/* parse l'expression */
			myParser.parseExpression(strData);
			isReady = !(myParser.hasError());
			return;

		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "initParser()", ex);}
		isReady = false;
	}
	
	
	public final void setVariable(final String variable, final double value) {if(myParser != null) myParser.addVariable(variable, new Double(value));}
	
	
	public final double yValue(double xValue)
	{
		try{
			if(isReady == true){
				if(myParser != null){
					myParser.addVariable("x", xValue);
					return myParser.getValue();
				}
			}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "yValue()", ex);}
		return Maths.NaN;
	}
	
	public final String getName() {return strName;}
	
	public final String getDescription() {return strDesc;}
	
	public final String getFunction() {return strFunc;}

	public final String toString() {return getDescription();}

	public final boolean isReady() {return isReady;}
	
	public final Color color() {return color;}
	
	
}
