/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package kernel.functions;

import java.awt.Color;

import kernel.Maths;
import kernel.Kernel;

public class PointsFunction implements _Function
{

	private String strName									= null;
	private String strDesc									= null;
	private String strFunc									= null;

	private boolean isReady									= false;
	private boolean blLogY									= false;
	
	private double[] xValues								= null;
	private double[] yValues								= null;
	
	private Color color										= null;
	



	
	
	
	public PointsFunction(final String strName, final String strDesc, final String strFunc, final Color color)
	{
		try{
			this.color = color;
			this.strName = strName;
			this.strDesc = strDesc;
			this.strFunc = strFunc;
			this.initParser(strFunc);
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "DistribFunction()", ex);}
	}
	
	

	

	private final void initParser(String strFunc)
	{
		try{
			
			if((strFunc != null) && (strFunc.length() >= FunctionsFactory.POINTS_FUNCTION.length())){
				
				String strData;
				
				if(strFunc.toLowerCase().startsWith(FunctionsFactory.LOG_POINTS_FUNCTION)){
					strData = strFunc.substring(FunctionsFactory.LOG_POINTS_FUNCTION.length() + 1, strFunc.length() - 1);
					blLogY = true;
				}else{
					strData = strFunc.substring(FunctionsFactory.POINTS_FUNCTION.length() + 1, strFunc.length() - 1);
					blLogY = false;
				}

				String[] tabl = strData.split("\\]\\[");
					
				xValues = new double[tabl.length];
				yValues = new double[tabl.length];
				String[] val;

				for(int i=0; i<tabl.length; i++){
					val = tabl[i].split(";");
					xValues[i] = Double.parseDouble(val[0]);
					if("NAN".equalsIgnoreCase(val[1])){
						yValues[i] = Maths.NaN;
					}else{
						yValues[i] = Double.parseDouble(val[1]);
					}

				}
				
				isReady = true;
				return;
			}

		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "initParser()", ex);}
		isReady = false;
		
	}

	
	
	
	public final double yValue(double xValue)
	{
		try{
			if(isReady == true){
				
				/* on recherche le bon x */
				int i=0;
				
				while((i<xValues.length) && (xValues[i] < xValue)) i++;
				
				/* si c'est avant on prend la premiere valeur */
				if(i == 0) return yValues[0];
				/* si c'est apres on prend la derniere valeur */
				if(i == xValues.length) return yValues[xValues.length-1];
				/* si une valeur d'a cote n'est pas un nombre, on calcule rien */
				if((yValues[i] == Maths.NaN) || (yValues[i-1] == Maths.NaN)) return Maths.NaN;
				
				if(blLogY == true){

					double x1 = Maths.log( xValues[i-1] );
					double x2 = Maths.log( xValues[i] );
					
					double y1 = Maths.log( yValues[i-1] );
					double y2 = Maths.log( yValues[i] );

					double a = (y2 - y1) / (x2 - x1);
					double b = y1 - a * x1;
					return Maths.pow( 10.0 , a * Maths.log( xValue ) + b );

				}else{
					
					double a = (yValues[i] - yValues[i-1]) / (xValues[i] - xValues[i-1]);
					
					double b = yValues[i] - a * xValues[i];
					
					return a * xValue + b;
					
				}
			}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "yValue()", ex);}
		return Maths.NaN;
	}
	

	public final String getName() {return strName;}
	
	public final String getDescription() {return strDesc;}
	
	public final String getFunction() {return strFunc;}

	public final String toString() {return getDescription();}

	public final boolean isReady() {return isReady;}
	
	public final Color color() {return color;}
	
	
}
