/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package simulation;

import java.io.Serializable;
import java.util.Vector;

import kernel.Kernel;
import kernel.functions.FunctionsFactory;
import ressources.Lang;
import simulation.components.Filter;
import simulation.components.ImageComponent;
import simulation.components.ImageMatrix;
import ui.simulation.SFrame;

public class SimulationSpec implements Serializable
{

	private static final long serialVersionUID 				= 1L;
	
	public static final int IMAGE_RGB						= 0;
	public static final int IMAGE_MONO						= 1;
	public static final int IMAGE_R							= 2;
	public static final int IMAGE_G							= 3;
	public static final int IMAGE_B							= 4;

	
	


	/* formule du bruit de fond */
	/*private String strBackNoise								= ">main: 1e26 * ( 5.61e-8 * %fnd% + 1.133e-8 * (266/5770)^4 * %zod% ) ;\n\n"
															+ ">autre: 1.2e26 * (3.5e-14*%zod% + 3.58e-8*%fnd%) ;\n\n"
															+ ">fnd: planck(%x%, 266) ;\n"
															+ ">zod: planck(%x%, 5770) ;\n\n"
															+ "trace: -20/10/4/1 ;";
	*/
	/* formule du bruit gaussien */
	/*private String strGaussianNoise							= ">main: gaussmcr( 500*%mu% ) ;\n\n"
															+ ">mu: sqrt( %sigma1%*%sigma1% + %sigma2%*%sigma2% ) ;\n\n"
															+ "sigma1: sqrt( %back_elec% + 1000 * %obs_time% + 100*100 ) ;\n"
															+ "sigma2: sqrt( %objects_elec% ) ;";

	
	

*/
	/* le filtre en monochromatique */
	private int iFilter										= 7;									
	



	/* memorise la fenetre des logs */
	private SFrame frame									= null;
	
	/* la matrice */
	private ImageMatrix matrix								= null;
	
	/* les composants de l'image */
	private Vector lstComponents							= new Vector();

	/* les filtres des observations */
	private Vector lstFilters	 							= new Vector();
	

	/* les ecouteurs */
	private Vector lstListeners								= new Vector();


	
	
	
	
	
	
	
	
	/* taille de l'image */
	public int PARAM_IMAGE_SIZE_X							= 160;
	public int PARAM_IMAGE_SIZE_Y							= 160;
	
	/* temps de pause */
	public double PARAM_OBSERVATION_TIME					= 3600;
 	
	/* resolution d'un pixel */
	public double PARAM_PIXEL_RESOLUTION					= 1.42;

	/* diametre du telescope */
	public double PARAM_TELESCOPE_DIAMETER					= 3.5;
	
 	/* facteur de zoom */
 	public double DISPLAY_ZOOM_VALUE						= 1.0;

 	/* dessine ou pas les positions des objets */
 	public boolean DISPLAY_OBJECTS_POSITIONS				= false;
 	

 	/* distribue ou pas les objets automatiquement */
 	public boolean DISTRIBUTION_AUTOMATIC					= true;
 	
	/* les fonctions de distributions */
 	public Vector DISTRIBUTION_FUNCTIONS					= new Vector();

	/* script de distribution des objets */
	public String DISTRIBUTION_SCRIPT						= "##\n"
															+ "## Script de distribution des objets\n"
															+ "## \n"
															+ "## Parcouru de haut en bas lors de l'étape de \n"
															+ "## distribution.\n"
															+ "##\n"
															+ "\n"
															+ "##\n"
															+ "## Voici comment placer un objet ponctuel\n"
															+ "# [ Intervale X ; Intervalle Y ; Nombre ; Intervalle Flux ]\n"
															+ "##\n"
															+ "## Exemples :\n"
															+ "# [ 50 ; 50 ; 1 ; 1e3 ]\n"
															+ "# [ 0..XMAX ; 0..YMAX ; 100 ; 1e3..1e4 ]\n"
															+ "\n"
															+ "##\n"
															+ "## Il est aussi possible d'utiliser des fonctions\n"
															+ "## de distribution plus complexes.\n"
															+ "spitzer2004\n";

	 
 	/* genere ou pas les objets automatiquement */
 	public boolean GENERATION_AUTOMATIC						= false;
 	
 	/* taille de la zone de geneartion d'un objet */
 	public int GENERATION_OBJECTS_SIZE						= 8;

 	
 	/* genere le bruit ou pas automatiquement */
 	public boolean NOISIFICATION_AUTOMATIC					= true;
 	
	/* formule de calcule du bruit */
 	public String NOISES_FORMULA							= "##\n"
 															+ "## Script de calcul des bruits\n"
 															+ "## \n"
 															+ "## Parcouru de bas en haut lors de l'étape \n"
 															+ "## d'ajout des bruits.\n"
 															+ "##\n"
 															+ "\n"
 															+ "## Bruit total\n"
 															+ "return: %bruit_fond% + %bruit_gaussien%\n"
															+ "\n"
															+ "## Bruit gaussien\n"
 															+ "bruit_gaussien: gaussmcr( %mu% )\n"
															+ "mu: sqrt( %sigma1% * %sigma1% + %sigma2% * %sigma2% )\n"
															+ "sigma1: sqrt( %bruit_fond% + 1000 * %obs_time% + 100*100 )\n"
															+ "sigma2: sqrt( %objects_elec% )\n"
 															+ "\n"
 															+ "## Bruit de fond\n"
 															+ "bruit_fond: fromJyPerSteradToElecPerPixel(%bruit_fond1%, %lambda%, %dlambda%, %obs_time%, %telesc_diam%, %pix_res%)\n"
 															+ "bruit_fond1: 1e26 * ( 5.61e-8 * %fnd% + 1.133e-8 * (266/5770)^4 * %zod% )\n"
 															+ "fnd: planck( %lambda%, 266 )\n"
 															+ "zod: planck( %lambda%, 5770 )\n";
 
 	
	/* valeur de normalisation */
 	public int NORMALISATION_VALUE							= 500;
 	
	/* puissance de normalisation */
 	public double NORMALISATION_POWER						= 1;
	
 	
 	
	/* largeur de mesure */
 	public int UTILS_INTENSITY_MEASURE_WIDTH				= 1;

	
 	
 	
 	
 	
	
	public SimulationSpec()
	{
		try{
			/* nouvelle matrice */
			matrix = new ImageMatrix(this); 
			
			/* ajoute les filtres */
			lstFilters.add(new Filter(Lang.STR_OPT_FILTER_U_DESC, 0.365E-6, 0.080E-6, Filter.FILTER_U));
			lstFilters.add(new Filter(Lang.STR_OPT_FILTER_B_DESC, 0.440E-6, 0.100E-6, Filter.FILTER_B));
			lstFilters.add(new Filter(Lang.STR_OPT_FILTER_V_DESC, 0.525E-6, 0.070E-6, Filter.FILTER_V));
			lstFilters.add(new Filter(Lang.STR_OPT_FILTER_R_DESC, 0.600E-6, 1.300E-6, Filter.FILTER_R));
			lstFilters.add(new Filter(Lang.STR_OPT_FILTER_I_DESC, 0.770E-6, 2.300E-6, Filter.FILTER_I));
			lstFilters.add(new Filter(Lang.STR_OPT_FILTER_SP11_DESC, 42.5E-6, 31.0E-6));
			lstFilters.add(new Filter(Lang.STR_OPT_FILTER_SP12_DESC, 79.0E-6, 42.0E-6));
			lstFilters.add(new Filter(Lang.STR_OPT_FILTER_SP13_DESC, 153.0E-6, 106.0E-6));
			lstFilters.add(new Filter(Lang.STR_OPT_FILTER_SP31_DESC, 32.5E-6, 15.0E-6));
			lstFilters.add(new Filter(Lang.STR_OPT_FILTER_SP32_DESC, 50.0E-6, 20.0E-6));
			lstFilters.add(new Filter(Lang.STR_OPT_FILTER_SP33_DESC, 80.0E-6, 40.0E-6));
			lstFilters.add(new Filter(Lang.STR_OPT_FILTER_SP34_DESC, 150.0E-6, 110.0E-6));
			lstFilters.add(new Filter(Lang.STR_OPT_FILTER_SPTZ_DESC, 24.0E-6, 4.8E-6));
			
			/* ajoute les fonctions de distributions */
			DISTRIBUTION_FUNCTIONS.add(FunctionsFactory.parse("spitzer2004", "Spitzer, Lagache G., Dole H., Puget J-L., et al, 2004", FunctionsFactory.LOG_POINTS_FUNCTION + "[0.9E-7;NaN][1.0E-7;1.475E9][1.0E-6;1.303E9][1.0E-5;6.500E8][1.0E-4;2.512E8][1.0E-3;8.695E7][1.0E-2;1.567E7][1.0E-1;2.5692E5][1.0;3.4378E3][1.0E1;1.0616E2][1.0E2;3.1950341][7.94E2;1.93859E-02][9.0E2;NaN]", null));
			DISTRIBUTION_FUNCTIONS.add(FunctionsFactory.parse("spitzer2005", "Spitzer, Pearson C., 2005 (to fix in good units)", FunctionsFactory.POINTS_FUNCTION + "[0.031;NaN][0.032;501187][0.1;1584893][0.32;3162277][1;1584893][3.2;1258925][10;1258925][32;1584893][100;1778279][125;NaN]", null));

		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "SimulationSpec()", ex);}
		
	}
	
	
	
	
	
	
	


	

	

	
	
	


	private final void fireSimulationStateChanged() {for(int i=0; i<lstListeners.size(); i++) try{ ((_SimulationSpecListener)(lstListeners.get(i))).onSimulationStateChanged(blSimulating); }catch(Exception ex) {}}

	public final void fireSimulationStartDistribution() {setSimulating(true); for(int i=0; i<lstListeners.size(); i++) try{ ((_SimulationSpecListener)(lstListeners.get(i))).onSimulationStartDistribution(); }catch(Exception ex) {}}
	public final void fireSimulationStartGeneration() {setSimulating(true); for(int i=0; i<lstListeners.size(); i++) try{ ((_SimulationSpecListener)(lstListeners.get(i))).onSimulationStartGeneration(); }catch(Exception ex) {}}
	public final void fireSimulationStartNoisation() {setSimulating(true); for(int i=0; i<lstListeners.size(); i++) try{ ((_SimulationSpecListener)(lstListeners.get(i))).onSimulationStartNoisation(); }catch(Exception ex) {}}
	public final void fireSimulationStartNormalization() {setSimulating(true); for(int i=0; i<lstListeners.size(); i++) try{ ((_SimulationSpecListener)(lstListeners.get(i))).onSimulationStartNormalization(); }catch(Exception ex) {}}
	
	public final void fireSimulationComponentsListChanged() {for(int i=0; i<lstListeners.size(); i++) try{ ((_SimulationSpecListener)(lstListeners.get(i))).onSimulationComponentsListChanged(); }catch(Exception ex) {}}
	public final void fireSimulationGeneratedComponentsChanged(int iGeneratedNumber) {for(int i=0; i<lstListeners.size(); i++) try{ ((_SimulationSpecListener)(lstListeners.get(i))).onSimulationGeneratedComponentsChanged(iGeneratedNumber); }catch(Exception ex) {}}
	
	public final void fireLog(final String strLog) {if(frame != null) frame.fireLog(strLog);}
	public final void fireLog(final String strLog, final boolean blRet) {if(frame != null) frame.fireLog(strLog, blRet);}

	
	
	
	

	
	
	
	/* memorise si une simulation est en cours */
	private boolean blSimulating 							= false;

	/* accesseurs */
	public final boolean isSimulating() {return blSimulating;}
	public final void setSimulating(boolean blSimulating)
	{
		if( this.blSimulating != blSimulating ){
			this.blSimulating = blSimulating;
			fireSimulationStateChanged();
		}
	}
	


	
	

	
	
	
	

	/* logs */
	public final SFrame getMainFrame() {return frame;}
	public final void setMainFrame(SFrame frame) {this.frame = frame;}

	/* composants de l'image */
	public synchronized final Vector getComponents() {return lstComponents;}
	public synchronized final void clearComponents() {lstComponents.clear();}
	public synchronized final void addComponent(ImageComponent component) {lstComponents.add(component);}
	public synchronized final void removeComponent(ImageComponent component) {lstComponents.remove(component);}

	/* matrice de l'image */
	public ImageMatrix getMatrix() {return matrix;}
	
	/* ecouteurs */
	public final void addListener(_SimulationSpecListener listener) {lstListeners.add(listener);}
	public final void removeListener(_SimulationSpecListener listener) {lstListeners.remove(listener);}
	

	
	
	
	
	/* les filtres des observations */
	public final Vector getFilters() {return lstFilters;}
	public final void setFilters(Vector lstFilters) {this.lstFilters = lstFilters;}

	/* index du filtre monochromatique */
	public final int getFilterIndexMono() {return iFilter;}
	public final void setFilterIndexMono(int iFilter) {this.iFilter = iFilter;}
	public final Filter getFilterMono() {return (Filter)(lstFilters.get(iFilter));}

	
	

}

