/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package simulation.components;

public class Filter
{
	
	public static final int FILTER_NOT_SPECIAL				= 0;
	public static final int FILTER_U						= 1;
	public static final int FILTER_B						= 2;
	public static final int FILTER_V						= 3;
	public static final int FILTER_R						= 4;
	public static final int FILTER_I						= 5;
	public static final int FILTER_TABLES_SIZES				= 6;

	private String strName									= null;
	
	private double lambdaCenterInMeter						= 0.0;
	private double dlambdaInMeter							= 0.0;
	private int iSpecialFilter								= FILTER_NOT_SPECIAL;
	
	
	
	public Filter(final String strName, final double lambdaCenterInMeter, final double dlambdaInMeter)
	{
		this.strName = strName;
		this.lambdaCenterInMeter = lambdaCenterInMeter;
		this.dlambdaInMeter = dlambdaInMeter;
	}
	
	public Filter(final String strName, final double lambdaCenterInMeter, final double dlambdaInMeter, final int iSpecialFilter)
	{
		this.strName = strName;
		this.lambdaCenterInMeter = lambdaCenterInMeter;
		this.dlambdaInMeter = dlambdaInMeter;
		this.iSpecialFilter = iSpecialFilter;
	}
	
	
	
	public final String getDescription() {return strName;}
	
	public String toString() {return getDescription();}
	
	

	public final double lambdaCenterInMeter() {return lambdaCenterInMeter;}
	public final double dlambdaInMeter() {return dlambdaInMeter;}

	
	
	public boolean isBlackFilter() {return (lambdaCenterInMeter <= 0.0);}
	public boolean isNotBlackFilter() {return (lambdaCenterInMeter > 0.0);}
	
	public boolean isSpecialFilterU() {return (iSpecialFilter == FILTER_U);}
	public boolean isSpecialFilterB() {return (iSpecialFilter == FILTER_B);}
	public boolean isSpecialFilterV() {return (iSpecialFilter == FILTER_V);}
	public boolean isSpecialFilterR() {return (iSpecialFilter == FILTER_R);}
	public boolean isSpecialFilterI() {return (iSpecialFilter == FILTER_I);}

	public boolean isSpecialFilter() {return (iSpecialFilter != FILTER_NOT_SPECIAL);}
	public boolean isNotSpecialFilter() {return (iSpecialFilter == FILTER_NOT_SPECIAL);}
	
	public final int specialFilterID() {return iSpecialFilter;}
	
	

}
