/*
 SOS Project Classes.

 Copyright (C) 2006, 2007 SOS Project <http://sos.project.free.fr>
 SOS Project is :
 Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
 Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package simulation.objects;

import kernel.Kernel;
import kernel.Maths;
import simulation.SimulationSpec;
import simulation.components.Filter;
import simulation.components.ImageComponent;

public class AiryPoint extends ImageComponent
{
	

	public AiryPoint(final SimulationSpec spec, final double xCoord, final double yCoord, final double flux)
	{
		super(spec, xCoord, yCoord, flux);
	}



	
	/**
	 * Calcule l'intensité en fonction de x et y.
	 * 
	 * @return
	 * 
	 */
	protected double calculateIntensity(final double x, final double y, final Filter filter)
	{
		try{

			double A = 0.0;
			double B = 0.0;
			double e = 0.0;

			final double lambda = filter.lambdaCenterInMeter();


			double r = Maths.euclidianDistance(0, 0, x, y);			// rayon du point considere
			final double Epsi = 0.01; 								// ouverture (0=claire, 0.33=Hubble)

			/* passage en radian */
			if(r == 0.0) r = 1.0E-9;
			r = Maths.fromArcsecToRadian(r * spec.PARAM_PIXEL_RESOLUTION);

			/* v (en radians) */
			double v = Math.PI * spec.PARAM_TELESCOPE_DIAMETER * r / lambda;
			//double v = 2.0 * Math.PI * 3.5 * r / lambda;

			/* epsilon au carre */
			double EpsiSquare = Epsi * Epsi;

			/* v multiplie par epsilon */
			double vTimesEpsi = v * Epsi;

			A = Math.pow(1.0 - EpsiSquare, 2.0);
			B = (2 * Maths.j1(v)) / v - (EpsiSquare * 2.0 * Maths.j1(vTimesEpsi)) / vTimesEpsi;
			e = B * B / A;

			if(e < 0) e = 0;

			return e;

		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "calculateIntensity()", ex);}
		return 0.0;
	}

}
