/*
 SOS Project Classes.

 Copyright (C) 2006, 2007 SOS Project <http://sos.project.free.fr>
 SOS Project is :
 Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
 Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package ui.common;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;

import kernel.Kernel;
import ressources.Lang;
import ui.UI;

public class DialogFrame extends JDialog implements WindowListener, ActionListener
{

	private static final long	serialVersionUID	= 1L;

	public static final int		BUTTONS_OK_CANCEL	= 1;
	public static final int		BUTTONS_CLOSE		= 2;
	public static final int		BUTTON_ERROR		= -1;
	public static final int		BUTTON_OK			= 1;
	public static final int		BUTTON_CANCEL		= 2;
	public static final int		BUTTON_CLOSE		= 3;

	private JButton				btnOK				= null;
	private JButton				btnCANCEL			= null;
	private JButton				btnCLOSE			= null;

	private int					iReturnValue		= 0;

	private ActionListener		l					= null;



	
	
	
	protected DialogFrame(final JFrame parent, final String strName, final boolean blModal) {this(parent, strName, blModal, parent.getLocationOnScreen().x + 20, parent.getLocationOnScreen().y + 20);}
	protected DialogFrame(final JFrame parent, final String strName, final boolean blModal, final int x, final int y)
	{
		super(parent, strName, blModal);
		try{
			this.getContentPane().setLayout(new BorderLayout());
			this.addWindowListener(this);
			this.setResizable(false);
			this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
			this.setLocation(x, y);
		}catch(Exception ex){
			Kernel.fireError(this.getClass().getName(), "DialogFrame()", ex);
		}
	}

	


	protected final void setContent(final JComponent content, final int iType)
	{
		try{
			JPanel pnl = UI.createPanel(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
			/* proprietes communes */
			c.fill = GridBagConstraints.BOTH;
			c.insets = new Insets(3, 3, 3, 3);
			c.gridy = 0;
			c.gridx = 0;
			c.weightx = 1.0;
			c.gridwidth = 1;
			/* ajoute une zone vide */
			pnl.add(UI.createPanel(null), c);
			c.weightx = 0.1;
			c.gridx = 1;
			/* ajoute les boutons */
			if(iType == BUTTONS_OK_CANCEL){
				/* ajoute le bouton annuler */
				pnl.add(btnCANCEL = UI.createButton("   " + Lang.STR_BTN_CANCEL + "   ", this, null), c);
				/* ajoute le bouton valider */
				c.gridx = 2;
				pnl.add(btnOK = UI.createButton("   " + Lang.STR_BTN_VALIDATE + "   ", this, null), c);
			}else{
				/* ajoute le bouton fermer */
				pnl.add(btnCLOSE = UI.createButton("   " + Lang.STR_BTN_CLOSE + "   ", this, null), c);
			}
			/* ajoute le tout */
			this.getContentPane().removeAll();
			this.getContentPane().add(content, BorderLayout.CENTER);
			this.getContentPane().add(pnl, BorderLayout.SOUTH);
			/* redimentionne */
			this.pack();
		}catch(Exception ex){
			Kernel.fireError(this.getClass().getName(), "setContent()", ex);
		}
	}



	public void setEnable(boolean bl)
	{
		try{
			/* on modifie les droits des boutons */
			if(btnCLOSE != null){
				/* juste le bouton fermer */
				btnCLOSE.setEnabled(bl);
			}else{
				/* les boutons valider annuler */
				btnOK.setEnabled(bl);
				btnCANCEL.setEnabled(bl);
			}
		}catch(Exception ex){
			Kernel.fireError(this.getClass().getName(), "setEnable()", ex);
		}
	}



	public boolean isEnable()
	{
		if(btnCLOSE != null) return btnCLOSE.isEnabled();
		else return btnOK.isEnabled();
	}



	public final int retournValue() {return iReturnValue;}



	private final void closeDialog(final int iReturnValue)
	{
		try{
			/* on memorise la valeur de retour */
			this.iReturnValue = iReturnValue;
			/* on cache la fenetre */
			this.setVisible(false);
			/* notifie */
			if(l != null) l.actionPerformed(new ActionEvent(this, iReturnValue, null));
		}catch(Exception ex){
			Kernel.fireError(this.getClass().getName(), "closeDialog()", ex);
		}
	}



	public void actionPerformed(ActionEvent ae)
	{
		try{
			if(ae.getSource() == btnOK){
				closeDialog(BUTTON_OK);
				return;
			}
			if(ae.getSource() == btnCANCEL){
				closeDialog(BUTTON_CANCEL);
				return;
			}
			if(ae.getSource() == btnCLOSE){
				closeDialog(BUTTON_CLOSE);
				return;
			}
		}catch(Exception ex){
			Kernel.fireError(this.getClass().getName(), "actionPerformed()", ex);
		}
	}



	public void windowClosing(WindowEvent e)
	{
		try{
			if(isEnable() == true){
				if(btnCLOSE != null) closeDialog(BUTTON_CLOSE);
				else closeDialog(BUTTON_CANCEL);
			}
		}catch(Exception ex){
			Kernel.fireError(this.getClass().getName(), "windowClosing()", ex);
		}
	}



	public void windowOpened(WindowEvent e) {}
	public void windowClosed(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowActivated(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
	

}
