/*
 SOS Project Classes.

 Copyright (C) 2006, 2007 SOS Project <http://sos.project.free.fr>
 SOS Project is :
 Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
 Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ui.common;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.util.Vector;

import javax.swing.JPanel;

import kernel.Maths;
import kernel.Kernel;
import kernel.functions._Function;

public class GraphCanvas extends JPanel
{

	private static final long	serialVersionUID	= 1L;

	public static final Color	COLOR_AXES			= new Color(20, 20, 20);
	public static final Color	COLOR_TEXT			= new Color(0, 0, 0);
	public static final Color	COLOR_BACK			= new Color(250, 250, 250);
	public static final Color[]	COLOR_FUNCS			= { new Color(250, 10, 10), new Color(10, 10, 255), new Color(10, 255, 10) };

	public static final int		SCALE_X_LINEAR		= 1;
	public static final int		SCALE_X_LOG			= 2;
	public static final int		SCALE_Y_LINEAR		= 4;
	public static final int		SCALE_Y_LOG			= 8;

	public static final int		SCALE_BOTH_LINEAR	= SCALE_X_LINEAR + SCALE_Y_LINEAR;
	public static final int		SCALE_BOTH_LOG		= SCALE_X_LOG + SCALE_Y_LOG;

	private Font				ftScale				= null;
	private Image				buffer				= null;

	private Dimension			dimAll				= null;
	private Rectangle			dimCurves			= null;
	private Graphics			g					= null;

	private Vector				funcs				= new Vector();
	private Vector				lines				= new Vector();

	private double				xMin				= 0.0;
	private double				xMax				= 0.0;
	private double				xStep1				= 0.0;
	private double				xStep2				= 0.0;

	private boolean				blFixedY			= false;

	private double				yMin				= 0.0;
	private double				yMax				= 0.0;
	private double				yStep1				= 0.0;
	private double				yStep2				= 0.0;
	
	private int					iScale				= SCALE_BOTH_LINEAR;

	private double				dblXScale			= 0.0;
	private double				dblYScale			= 0.0;



	public GraphCanvas(final double xMin, final double xMax, final double xStep1, final double xStep2)
	{
		this.ftScale = new Font(this.getFont().getName(), Font.PLAIN, 9);
		this.setXPolicy(xMin, xMax, xStep1, xStep2);
	}



	public final void setXPolicy(final double xMin, final double xMax, final double xStep1, final double xStep2)
	{
		this.xMin = xMin;
		this.xMax = xMax;
		this.xStep1 = xStep1;
		this.xStep2 = xStep2;
		this.paintAllGraphic();
		this.repaint();
	}



	public final void setYPolicy(final double yMin, final double yMax, final double yStep1, final double yStep2)
	{
		this.yMin = yMin;
		this.yMax = yMax;
		this.yStep1 = yStep1;
		this.yStep2 = yStep2;
		this.blFixedY = true;
		this.paintAllGraphic();
		this.repaint();
	}



	public final void setYPolicy()
	{
		this.blFixedY = false;
		this.paintAllGraphic();
		this.repaint();
	}



	public final void clearFunctions()
	{
		this.funcs.clear();
		this.lines.clear();
		this.paintAllGraphic();
		this.repaint();
	}



	public final void addFunction(final _Function func)
	{
		this.funcs.add(func);
		this.paintAllGraphic();
		this.repaint();
	}



	public final void setScale(final int iScale)
	{
		this.iScale = iScale;
		this.paintAllGraphic();
		this.repaint();
	}



	public final void addYAxe(final double yvalue)
	{
		this.lines.add(new Double(yvalue));
		this.paintAllGraphic();
		this.repaint();
	}



	private final void paintAllGraphic()
	{
		try{
			/* demande les dimensions */
			dimAll = getSize();
			/* on regarde si la zone est valide */
			if(dimAll.width > 0 && dimAll.height > 0){
				/* dimension de la zone pour les courbes */
				// dimCurves = new Rectangle(25, 3, dimAll.width - 33,
				// dimAll.height - 20);
				dimCurves = new Rectangle(30, 5, dimAll.width - 40, dimAll.height - 20);
				/* crée le buffer */
				buffer = createImage(dimAll.width, dimAll.height);
				/* demande le graphic */
				g = buffer.getGraphics();
				g.setFont(ftScale);
				/* on trace le fond */
				this.paintBack();
				/* courbes */
				if(funcs.size() == 0){
					/* facteur d'echelle en x */
					dblXScale = (xMax - xMin) / ((double)(dimCurves.width));
					/* aucune courbe */
					g.setColor(COLOR_TEXT);
					g.drawString("Aucune donnée", dimCurves.x + 6, dimCurves.y + 14);
				}else{
					/* facteur d'echelle en x */
					dblXScale = (xMax - xMin) / ((double)(dimCurves.width));
					/* on calcule les données */
					Vector[] data = new Vector[funcs.size()];
					for(int i = 0; i < funcs.size(); i++)
						data[i] = calculateValues((_Function)(funcs.get(i)), i == 0);
					/* facteur d'echelle en y */
					dblYScale = (yMax - yMin) / ((double)(dimCurves.height));
					/* on paint les courbes */
					for(int i = 0; i < data.length; i++)
						if(data[i] != null) paintCurve(data[i], getColor((_Function)(funcs.get(i)), i));
					/* on calcule les graduations */
					if(blFixedY == false){
						yStep1 = (yMax - yMin) / 5;
						yStep2 = (yMax - yMin) / 50;
					}
				}
				/* on paint les graduations */
				paintAxes();
			}
		}catch(Exception ex){
			Kernel.fireError(this.getClass().getName(), "paintAllGraphic()", ex);
		}
	}



	private final Vector calculateValues(final _Function func, final boolean blFirst)
	{
		Vector data = null;
		try{
			/* position en pixels sur x */
			int xAbsolute;
			/* valeurs des positions reelles */
			double xActual = xMin;
			double yActual = 0.0;
			/* points */
			data = new Vector(dimCurves.width);
			/* on calcule les points */
			for(xAbsolute = 0; xAbsolute < dimCurves.width; xAbsolute++){
				/* on calcul la valeur du point */
				xActual = xActual + dblXScale;
				/* on regarde quelle est la valeur */
				if((iScale & SCALE_X_LOG) == SCALE_X_LOG) yActual = func.yValue(Maths.pow(10, xActual));
				else yActual = func.yValue(xActual);
				if((iScale & SCALE_Y_LOG) == SCALE_Y_LOG) yActual = Maths.log(yActual);
				/* on ajoute le point */
				data.add(new Double(yActual));
				/* on cherche la valeur max et min */
				if(blFixedY == false){
					if((xAbsolute == 0) && (blFirst == true)){
						yMin = yActual;
						yMax = yActual;
					}else{
						if(yActual < yMin) yMin = yActual;
						if(yActual > yMax) yMax = yActual;
					}
				}
			}
		}catch(Exception ex){
			Kernel.fireError(this.getClass().getName(), "calculateValues()", ex);
		}
		return data;
	}



	private final void paintBack()
	{
		try{
			/* efface le fond */
			g.clearRect(0, 0, dimAll.width, dimAll.height);
			// g.setColor(COLOR_AXES);
			// g.drawRect(0, 0, dimAll.width-1, dimAll.height-1);
			/* zone du graphique */
			g.setColor(COLOR_BACK);
			g.fillRect(dimCurves.x, dimCurves.y, dimCurves.width, dimCurves.height);
			/* tour de la zone du graphique */
			g.setColor(COLOR_AXES);
			g.drawRect(dimCurves.x, dimCurves.y, dimCurves.width, dimCurves.height);
		}catch(Exception ex){
			Kernel.fireError(this.getClass().getName(), "paintBack()", ex);
		}
	}



	private final void paintAxes()
	{
		try{
			int xAbsolute, yAbsolute, iPosx;
			g.setColor(COLOR_AXES);
			/* axes en y */
			int x;
			double val;
			for(int i = 0; i < lines.size(); i++){
				val = ((Double)(lines.get(i))).doubleValue();
				x = dimCurves.x + (int)((val - xMin) / dblXScale + 0.49);
				g.drawLine(x, dimCurves.y, x, dimCurves.y + dimCurves.height);
			}
			/* trace les graduations en x */
			if(xStep1 > 0.0){
				for(double k = xMin; k <= xMax; k += xStep1){
					/* echelle principale */
					xAbsolute = dimCurves.x + (int)((k - xMin) / dblXScale + 0.49);
					g.drawLine(xAbsolute, dimCurves.y + dimCurves.height - 4, xAbsolute, dimCurves.y + dimCurves.height);
					g.drawLine(xAbsolute, dimCurves.y, xAbsolute, dimCurves.y + 4);
					/* valeur */
					g.drawString(Maths.toString(k, 1), xAbsolute - 4, dimCurves.y + dimCurves.height + 11);
					if((iScale & SCALE_X_LOG) == SCALE_X_LOG) g.drawString("10", xAbsolute + 2 - 18, dimCurves.y + dimCurves.height + 11 + 4);
					/* echelle secondaire */
					if(xStep2 > 0.0 && k != xMax){
						for(double l = k + xStep2; l < k + xStep1; l += xStep2){
							/* echelle secondaire */
							xAbsolute = dimCurves.x + (int)((l - xMin) / dblXScale + 0.49);
							if(xAbsolute < dimCurves.x + dimCurves.width){
								g.drawLine(xAbsolute, dimCurves.y + dimCurves.height - 2, xAbsolute, dimCurves.y + dimCurves.height);
								g.drawLine(xAbsolute, dimCurves.y, xAbsolute, dimCurves.y + 2);
							}
						}
					}
				}
			}
			/* trace les graduations en y */
			if(yStep1 > 0.0){
				for(double k = yMin; k <= yMax; k += yStep1){
					/* echelle principale */
					yAbsolute = dimCurves.height + dimCurves.y - (int)((k - yMin) / dblYScale + 0.49);
					g.drawLine(dimCurves.x, yAbsolute, dimCurves.x + 4, yAbsolute);
					g.drawLine(dimCurves.x + dimCurves.width - 4, yAbsolute, dimCurves.x + dimCurves.width, yAbsolute);
					/* valeur */
					if((iScale & SCALE_Y_LOG) == SCALE_Y_LOG){
						iPosx = dimCurves.x - g.getFontMetrics().stringWidth("10" + Maths.toString(k, 1)) - 1;
						if(iPosx < 0) iPosx = 0;
						g.drawString("10", iPosx, yAbsolute + dimCurves.y + 1);
						g.drawString(Maths.toString(k, 1), iPosx + g.getFontMetrics().stringWidth("10"), yAbsolute + dimCurves.y - 3);
					}else{
						iPosx = dimCurves.x - g.getFontMetrics().stringWidth(Maths.toString(k, 1)) - 1;
						if(iPosx < 0) iPosx = 0;
						g.drawString(Maths.toString(k, 1), iPosx, yAbsolute + dimCurves.y - 3);
					}
					/* echelle secondaire */
					if(yStep2 > 0.0 && k != yMax){
						for(double l = k + yStep2; l < k + yStep1; l += yStep2){
							/* echelle secondaire */
							yAbsolute = dimCurves.height + dimCurves.y - (int)((l - yMin) / dblYScale + 0.49);
							if(yAbsolute > dimCurves.y){
								g.drawLine(dimCurves.x, yAbsolute, dimCurves.x + 2, yAbsolute);
								g.drawLine(dimCurves.x + dimCurves.width - 2, yAbsolute, dimCurves.x + dimCurves.width, yAbsolute);
							}
						}
					}
				}
			}
		}catch(Exception ex){
			Kernel.fireError(this.getClass().getName(), "paintAxes()", ex);
		}
	}



	private final void paintCurve(final Vector data, final Color color)
	{
		try{
			/* positions en pixels */
			int xAbsolute;
			int yAbsolute;
			int yLastAbsolute = 0;
			/* valeur reelle */
			double yActual = 0.0;
			/* on trace les points */
			g.setColor(color);
			yLastAbsolute = Integer.MIN_VALUE;
			for(xAbsolute = 0; xAbsolute < dimCurves.width; xAbsolute++){
				/* on recupere la valeur */
				yActual = ((Double)(data.get(xAbsolute))).doubleValue();
				if(yActual != Double.NEGATIVE_INFINITY){
					yAbsolute = dimCurves.height + ((int)(((yMin - yActual) / dblYScale) - 0.49));
					/* trace le point */
					if(yLastAbsolute != Integer.MIN_VALUE) g.drawLine(xAbsolute - 1 + dimCurves.x, yLastAbsolute + dimCurves.y, xAbsolute + dimCurves.x, yAbsolute + dimCurves.y);
				}else{
					yAbsolute = Integer.MIN_VALUE;
				}
				/* memorise la valeur */
				yLastAbsolute = yAbsolute;
			}
		}catch(Exception ex){
			Kernel.fireError(this.getClass().getName(), "paintCurve()", ex);
		}
	}



	private final Color getColor(final _Function func, final int index)
	{
		if(func.color() != null) return func.color();
		else return COLOR_FUNCS[index % COLOR_FUNCS.length];
	}



	public void paint(Graphics g)
	{
		try{
			if(dimAll == null) return;
			/* on regarde si les dimension on changées */
			boolean changedDimensions = !dimAll.equals(getSize());
			/* on regarde si on doit redessiner les courbes */
			if((buffer == null) || (changedDimensions == true)) paintAllGraphic();
			/* on dessine le buffer */
			g.drawImage(buffer, 0, 0, null);
		}catch(Exception ex){
			Kernel.fireError(this.getClass().getName(), "paint()", ex);
		}
	}

}
