/*
 SOS Project Classes.

 Copyright (C) 2006, 2007 SOS Project <http://sos.project.free.fr>
 SOS Project is :
 Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
 Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ui.simulation;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import kernel.Kernel;
import ressources.Lang;
import simulation.SimulationSpec;
import ui.UI;
import ui.simulation.editor.Editor;
import ui.simulation.editor.controlbar.ControlBar;
import ui.simulation.frames.AboutDialog;
import ui.simulation.frames.GenerationDialog;
import ui.simulation.frames.InformationDialog;
import ui.simulation.frames.IntensityUtilDialog;
import ui.simulation.frames.NoisesDialog;
import ui.simulation.frames.NormalizationDialog;
import ui.simulation.frames.ObjectsDialog;
import ui.simulation.frames.PropertiesDialog;

public class SFrame extends JFrame implements WindowListener, ActionListener
{

	private static final long serialVersionUID 			= 1L;
	
	private JTabbedPane tab								= null;
	
	private SimulationSpec simul						= null;
	private Editor editor								= null;

	private JTextArea txtLogs							= null;
	private JScrollPane scrollLogs						= null;
	private JPanel pnlLogs								= null;
	private JButton btnLogs								= null;

	private SMenu menu									= null;

	
	
	/* la fenetre de generation */
	private PropertiesDialog frameProp					= null;
	private ObjectsDialog frameObj						= null;
	private GenerationDialog frameGen					= null;
	private NoisesDialog frameNoises					= null;
	private NormalizationDialog frameNorm				= null;
	private InformationDialog frameInfos				= null;
	private IntensityUtilDialog frameIntens				= null;
	

	
	
	
	public SFrame()
	{
		super(Lang.STR_SIMUL_FRAME_TITLE);
		try{
			//UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
			buildContain();
			toggleLogs();
			Kernel.setLogger(txtLogs);
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "SFrame()", ex);}
	}
	
	
	
	
	
	
	
	
	private final void buildContain()
	{
		try{
			/* tab */
			tab = new JTabbedPane();
			
			/* zone de texte */
			txtLogs = new JTextArea("");
			txtLogs.setEditable(false);
			
			/* zone scrollable */
			scrollLogs = new JScrollPane(txtLogs);
			scrollLogs.setBorder(null);
			scrollLogs.setPreferredSize(new Dimension(140, 140));
			
			/* zone de commande */
			JPanel pnlCom = UI.createPanel(new BorderLayout());
			pnlCom.add(btnLogs = UI.createButton("-", this, new Dimension(18, 18)), BorderLayout.WEST);			
			pnlCom.add(UI.createLabel(" Console", UI.FONT_BOLD), BorderLayout.CENTER);
			pnlLogs = UI.createPanel(new BorderLayout());
			pnlLogs.add(pnlCom, BorderLayout.NORTH);
			
			
			menu = new SMenu(this);
			this.setJMenuBar(menu);
			menu.setMenuState(false);
			
			
			
			/* ajoute le tout */
			this.getContentPane().setLayout(new BorderLayout());
			this.getContentPane().add(tab, BorderLayout.CENTER);
			this.getContentPane().add(pnlLogs, BorderLayout.SOUTH);
			this.addWindowListener(this);
			this.pack();
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "buildContain()", ex);}
	}


	
	
	
	
	

	
	
	
	
	
	
	
	
	
	public final void quitSimulation()
	{
		try{
			
			/* on ferme la simulation en cours */
			if(closeSimulation() == false) return;
			
			/* on cache la fenetre */
			this.setVisible(false);
			
			/* on detruit la fenetre */
			this.dispose();
			
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "quitSimulation()", ex);}
	}

	public final boolean closeSimulation()
	{
		try{
			
			if(simul == null) return true;
			
			/* on enleve tous les onglets */
			this.tab.removeAll();
			
			/* on change les menus */
			menu.setMenuState(false);
			
			/* on supprime la simulation actuelle */
			this.simul = null;
			this.editor = null;
			
			/* on ferme les fenetres */
			if(frameObj != null) frameObj.setVisible(false);
			if(frameGen != null) frameGen.setVisible(false);
			if(frameNorm != null) frameNorm.setVisible(false);
			if(frameNoises != null) frameNoises.setVisible(false);
			if(frameInfos != null) frameInfos.setVisible(false);
			if(frameIntens != null) frameIntens.setVisible(false);
			frameProp = null;
			frameObj = null;
			frameGen = null;
			frameNorm = null;
			frameNoises = null;
			frameInfos = null;
			frameIntens = null;
			
			/* on nettoie la memoire */
			System.gc();
			
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "closeSimulation()", ex);}
		return true;
	}

	public final void loadSimulation(final SimulationSpec specInit, final boolean blWizard)
	{
		try{

			/* on regarde s'il faut fermer la simulation en cours */
			if(closeSimulation() == false) return;

			/* simulation */
			if(specInit == null) this.simul = new SimulationSpec();
			else this.simul = specInit;
			this.simul.setMainFrame(this);
			
			/* on change les menus */
			menu.setMenuState(true);
			
			/* on ajoute l'editeur */
			this.editor = new Editor(simul);
			this.tab.addTab("Editeur", editor);
			
			/* on charge les fenetres */
			frameProp = new PropertiesDialog(simul);
			frameObj = new ObjectsDialog(simul);
			frameGen = new GenerationDialog(simul);
			frameNorm = new NormalizationDialog(simul);
			frameNoises = new NoisesDialog(simul);
			frameInfos = new InformationDialog(simul);
			frameIntens = new IntensityUtilDialog(simul);

			/* on affiche les proprietes si c'est une nouvelle simulation */
			if((specInit == null) && (blWizard == true)) showPropertiesDialog();
			
			windowsOrganize();

		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "loadSimulation()", ex);}
	}
	
	
	
	
	
	public final void saveImage(final String strFormat) {Kernel.saveImage(simul.getMatrix().getImage(), strFormat, this);}
	public final void saveSimulation() {Kernel.saveSimulation(simul, this);}
	
	
	
	
	
	
	
	public final void windowsOrganize()
	{
		try{
			if(frameProp != null) {frameProp.setLocation(this.getLocationOnScreen().x+10, this.getLocationOnScreen().y+ControlBar.BAR_DIMENSION.height+74);}
			
			/* colone de gauche */
			if(frameObj != null){
				frameObj.setSize(340, 410);
				frameObj.setLocation(editor.getImageCanvas().getLocationOnScreen().x+5, editor.getImageCanvas().getLocationOnScreen().y+5);
			}
			if(frameGen != null){
				frameGen.setLocation(editor.getImageCanvas().getLocationOnScreen().x+5, editor.getImageCanvas().getLocationOnScreen().y+10+410);
			}
			/* colone de droite */
			if(frameNoises != null){
				frameNoises.setSize(340, 410);
				frameNoises.setLocation(editor.getImageCanvas().getLocationOnScreen().x+editor.getImageCanvas().getWidth()-frameNoises.getWidth()-5, editor.getImageCanvas().getLocationOnScreen().y+5);
			}
			if(frameNorm != null){
				frameNorm.setLocation(editor.getImageCanvas().getLocationOnScreen().x+editor.getImageCanvas().getWidth()-frameNorm.getWidth()-5, editor.getImageCanvas().getLocationOnScreen().y+10+410);
			}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "windowsOrganize()", ex);}
	}
	
	public final void windowsShowAll()
	{
		try{
			if(frameObj != null) frameObj.setVisible(true);
			if(frameGen != null) frameGen.setVisible(true);
			if(frameNoises != null) frameNoises.setVisible(true);
			if(frameNorm != null) frameNorm.setVisible(true);
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "windowsShowAll()", ex);}
	}
	
	public final void windowsHideAll()
	{
		try{
			if(frameObj != null) frameObj.setVisible(false);
			if(frameGen != null) frameGen.setVisible(false);
			if(frameNoises != null) frameNoises.setVisible(false);
			if(frameNorm != null) frameNorm.setVisible(false);
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "windowsHideAll()", ex);}
	}
	
	

	
	
	
	
	

	
	public final Editor editor() {return editor;}
	
	public final void fireLog(final String strMessage) {fireLog(strMessage, true);}

	public final void fireLog(final String strMessage, final boolean blRetour)
	{
		try{
			
			/* informations de débuggage */
			if(Kernel.DEBUG_SIMUL) System.out.println(strMessage);
			
			if(txtLogs == null) return;
			
			/* affiche le texte */
			if(blRetour == true) txtLogs.setText(txtLogs.getText() + "\n" + strMessage);
			else txtLogs.setText(txtLogs.getText() + strMessage);
			/* met à jour */
			SwingUtilities.invokeLater(
				new Runnable()
				{
					public void run()
					{
						try{
							txtLogs.updateUI();
							txtLogs.scrollRectToVisible(new Rectangle(1, txtLogs.getHeight()+100, 1, 1));
						}catch(Exception ex) {}
					}
				}
			);
		}catch(Exception ex) {}
	}
	
	private final void toggleLogs()
	{
		try{
			if("+".equals(btnLogs.getText()) == true){
				/* on doit afficher les logs */
				pnlLogs.add(scrollLogs, BorderLayout.CENTER);
				btnLogs.setText("-");
			}else{
				/* on doit cacher les logs */
				pnlLogs.remove(scrollLogs);
				btnLogs.setText("+");
			}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "toggleLogs()", ex);}
	}
	
	
	


	
	
	
	

	
	
	
	
	
	
	
	

	
	public final void showPropertiesDialog() {if(frameProp != null) frameProp.setVisible(true);}
	
	public final void showDistributionDialog(final boolean blForceVisible) {if(frameObj != null) frameObj.setVisible(blForceVisible?true:(!frameObj.isVisible()));}
	
	public final void showGenerationDialog(final boolean blForceVisible) {if(frameGen != null) frameGen.setVisible(blForceVisible?true:(!frameGen.isVisible()));}
	
	public final void showNormalizationDialog(final boolean blForceVisible) {if(frameNorm != null) frameNorm.setVisible(blForceVisible?true:(!frameNorm.isVisible()));}
	
	public final void showNoisesDialog(final boolean blForceVisible) {if(frameNoises != null) frameNoises.setVisible(blForceVisible?true:(!frameNoises.isVisible()));}

	public final void showInformationDialog(final boolean blForceVisible) {if(frameInfos != null) frameInfos.setVisible(blForceVisible?true:(!frameInfos.isVisible()));}
	
	public final void showIntensityUtilDialog(final boolean blForceVisible) {if(frameIntens != null) frameIntens.setVisible(blForceVisible?true:(!frameIntens.isVisible()));}
	
	public final void showAboutDialog() {new AboutDialog(this).setVisible(true);}
	
	


	
	
	
	
	
	
	
	
	
	
	
	
	
	
	public final void actionPerformed(ActionEvent ae)
	{
		try{
			if(ae.getSource() == btnLogs) {toggleLogs(); return;}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}
	

	

	public void windowOpened(WindowEvent e) {}
	public void windowClosing(WindowEvent e) {quitSimulation();}
	public void windowClosed(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowActivated(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}

	
}
