/*
 SOS Project Classes.

 Copyright (C) 2006, 2007 SOS Project <http://sos.project.free.fr>
 SOS Project is :
 Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
 Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ui.simulation;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import kernel.Kernel;
import ressources.Icons;
import ui.UI;

public class SMenu extends JMenuBar implements ActionListener
{

	private static final long	serialVersionUID			= 1L;

	private SFrame				frame						= null;

	private JMenu				mnuSimulation				= null;
	private JMenu				mnuStages					= null;
	private JMenu				mnuUtils					= null;
	private JMenu				mnuWindows					= null;
	private JMenu				mnuAbout					= null;
	private JMenu 				mnuExportImage				= null;

	private JMenuItem			mnuSimulationNew			= null;
	private JMenuItem			mnuSimulationClose			= null;
	private JMenuItem			mnuSimulationQuit			= null;
	private JMenuItem			mnuSimulationExportAsTXT	= null;
	private JMenuItem			mnuStagesProperties			= null;
	private JMenuItem			mnuStagesNormalization		= null;
	private JMenuItem			mnuStagesDistribution		= null;
	private JMenuItem			mnuStagesGeneration			= null;
	private JMenuItem			mnuStagesNoisification		= null;
	private JMenuItem			mnuUtilsIntensity			= null;
	private JMenuItem			mnuUtilsInformation			= null;
	private JMenuItem			mnuWindowsShowAll			= null;
	private JMenuItem			mnuWindowsHideAll			= null;
	private JMenuItem			mnuWindowsOrganize			= null;
	private JMenuItem			mnuAboutInformation			= null;


	
	

	public SMenu(final SFrame frame)
	{
		try{
			this.frame = frame;
			/* menu simulation */
			mnuSimulation = new JMenu("Simulation");
			mnuSimulationNew = UI.createMenuItem("Nouvelle", this, null);
			mnuSimulation.add(mnuSimulationNew);
			mnuSimulationClose = UI.createMenuItem("Fermer", this, null);
			mnuSimulation.add(mnuSimulationClose);
			mnuSimulation.addSeparator();
			mnuExportImage = new JMenu("Exporter l'image");
			String formatNames[] = ImageIO.getWriterFormatNames();
			Vector formats = new Vector();
			JMenuItem item;
			String strFormat;
			for(int i = 0; i<formatNames.length; i++){
				strFormat = formatNames[i].toUpperCase();
				if(formats.indexOf(strFormat) < 0){
					formats.add(strFormat);
					item = UI.createMenuItem("Fichier "+strFormat, this, null);
					item.setActionCommand(strFormat);
					mnuExportImage.add(item);
				}
			}
			mnuSimulation.add(mnuExportImage);
			mnuSimulationExportAsTXT = UI.createMenuItem("Exporter au format TXT", this, null);
			mnuSimulation.add(mnuSimulationExportAsTXT);
			mnuSimulation.addSeparator();
			mnuSimulationQuit = UI.createMenuItem("Quitter", this, null);
			mnuSimulation.add(mnuSimulationQuit);
			this.add(mnuSimulation);
			/* menu etapes */
			mnuStages = new JMenu("Etapes");
			mnuStagesProperties = UI.createMenuItem("Propriétés...", this, Icons.MENU_PROPERTIES);
			mnuStages.add(mnuStagesProperties);
			mnuStages.addSeparator();
			mnuStagesDistribution = UI.createMenuItem("Objets...", this, Icons.MENU_MANAGE_OBJECTS);
			mnuStages.add(mnuStagesDistribution);
			mnuStagesGeneration = UI.createMenuItem("Générer...", this, Icons.MENU_GENERATE_OBJECTS);
			mnuStages.add(mnuStagesGeneration);
			mnuStagesNoisification = UI.createMenuItem("Bruits...", this, Icons.MENU_BACK_NOISE);
			mnuStages.add(mnuStagesNoisification);
			mnuStagesNormalization = UI.createMenuItem("Normalisation...", this, Icons.MENU_NORMALIZE);
			mnuStages.add(mnuStagesNormalization);
			this.add(mnuStages);
			/* menu outils */
			mnuUtils = new JMenu("Outils");
			// mnuUtilsDistribution = UI.createMenuItem("Distribution...", this,
			// Icons.MENU_DISTRIB_STATS); mnuUtils.add(mnuUtilsDistribution);
			mnuUtilsIntensity = UI.createMenuItem("Intensité...", this, Icons.MENU_INTENS_STATS);
			mnuUtils.add(mnuUtilsIntensity);
			mnuUtils.addSeparator();
			mnuUtilsInformation = UI.createMenuItem("Informations...", this, Icons.MENU_INFOS);
			mnuUtils.add(mnuUtilsInformation);
			this.add(mnuUtils);
			/* menu fenetres */
			mnuWindows = new JMenu("Fenêtres");
			mnuWindowsShowAll = UI.createMenuItem("Toutes visibles", this, null);
			mnuWindows.add(mnuWindowsShowAll);
			mnuWindowsHideAll = UI.createMenuItem("Toutes cachées", this, null);
			mnuWindows.add(mnuWindowsHideAll);
			mnuWindows.addSeparator();
			mnuWindowsOrganize = UI.createMenuItem("Organiser", this, null);
			mnuWindows.add(mnuWindowsOrganize);
			this.add(mnuWindows);
			/* menu aide */
			mnuAbout = new JMenu(" ? ");
			mnuAboutInformation = UI.createMenuItem("A propos...", this, Icons.MENU_ABOUT);
			mnuAbout.add(mnuAboutInformation);
			// mAbout.addSeparator();
			// mnuAboutHelp = UI.createMenuItem("Aide...", this, null);
			// mAbout.add(mnuAboutHelp);
			this.add(mnuAbout);
		}catch(Exception ex){
			Kernel.fireError(this.getClass().getName(), "SMenu()", ex);
		}
	}



	public final void setMenuState(final boolean active)
	{
		try{
			mnuSimulationNew.setEnabled(!active);
			mnuSimulationClose.setEnabled(active);
			mnuExportImage.setEnabled(active);
			mnuSimulationExportAsTXT.setEnabled(active);
			mnuStages.setEnabled(active);
			mnuWindows.setEnabled(active);
			mnuUtils.setEnabled(active);
		}catch(Exception ex){
			Kernel.fireError(this.getClass().getName(), "setMenuState()", ex);
		}
	}



	public final void actionPerformed(ActionEvent ae)
	{
		try{
			/* menu simulation */
			if(ae.getSource() == mnuSimulationNew) {frame.loadSimulation(null, true);return;}
			if(ae.getSource() == mnuSimulationClose) {frame.closeSimulation(); return;}
			if(ae.getSource() == mnuSimulationQuit) {frame.quitSimulation(); return;}
			if(ae.getSource() == mnuSimulationExportAsTXT) {frame.saveSimulation(); return;}
			/* menu etapes */
			if(ae.getSource() == mnuStagesProperties) {frame.showPropertiesDialog(); return;}
			if(ae.getSource() == mnuStagesDistribution) {frame.showDistributionDialog(true); return;}
			if(ae.getSource() == mnuStagesGeneration) {frame.showGenerationDialog(true); return;}
			if(ae.getSource() == mnuStagesNoisification) {frame.showNoisesDialog(true); return;}
			if(ae.getSource() == mnuStagesNormalization) {frame.showNormalizationDialog(true); return;}
			/* menu outils */
			if(ae.getSource() == mnuUtilsIntensity) {frame.showIntensityUtilDialog(true); return;}
			if(ae.getSource() == mnuUtilsInformation) {frame.showInformationDialog(true); return;}
			/* menu fenetres */
			if(ae.getSource() == mnuWindowsShowAll) {frame.windowsShowAll(); return;}
			if(ae.getSource() == mnuWindowsHideAll) {frame.windowsHideAll(); return;}
			if(ae.getSource() == mnuWindowsOrganize) {frame.windowsOrganize(); return;}
			/* menu aide */
			if(ae.getSource() == mnuAboutInformation) {frame.showAboutDialog(); return;}
			
			if(ae.getSource() instanceof JMenuItem){
				JMenuItem item = (JMenuItem)(ae.getSource());
				if((item.getActionCommand() != null) && (item.getActionCommand().length() > 0)){
					frame.saveImage(item.getActionCommand().toLowerCase());
					return;
				}
			}			
			
		}catch(Exception ex){
			Kernel.fireError(this.getClass().getName(), "actionPerformed()", ex);
		}
	}

}
