/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package ui.simulation.editor.controlbar;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;

import kernel.Kernel;
import ressources.Icons;
import simulation.SimulationSpec;
import simulation._SimulationSpecListener;

public class ControlBar extends JPanel implements ActionListener, _SimulationSpecListener
 {

	private static final long serialVersionUID			= 1L;
	
	public static final Dimension GAP_DIMENSION			= new Dimension(4, 4);
	public static final Dimension STAGE_DIMENSION		= new Dimension(90, 42);
	public static final Dimension INFOS_DIMENSION		= new Dimension(170, 46);
	public static final Dimension BAR_DIMENSION			= new Dimension(60, 50);
	
	public static final int STAGE_PROPERTIES			= 0;
	public static final int STAGE_OBJECTS				= 1;
	public static final int STAGE_GENERATION			= 2;
	public static final int STAGE_NOISES				= 3;
	public static final int STAGE_NORMALIZATION			= 4;
	public static final int STAGE_STUDIES				= 5;

	public static final int STATE_DONE					= 0;
	public static final int STATE_CURR					= 1;
	public static final int STATE_WAIT					= 2;
	
	private SimulationSpec simulSpec					= null;
	
	private Stage[] stages								= new Stage[6];
	
	private int iActualStage							= STAGE_PROPERTIES;

	
	
	
 	public ControlBar(final SimulationSpec simulSpec)
 	{
 		super(null);
 		try{
 			this.simulSpec = simulSpec;
 			this.simulSpec.addListener(this);
 			this.setOpaque(false);
 			this.setMaximumSize(BAR_DIMENSION);
 			this.setMinimumSize(BAR_DIMENSION);
 			this.setPreferredSize(BAR_DIMENSION);
 			this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
	 		/* on construit les elements graphiques */
 			this.add(Box.createRigidArea(GAP_DIMENSION));
 			this.add(stages[STAGE_PROPERTIES] = new Stage(this, "Parametres", Icons.ICON_PROPERTIES_0, Icons.ICON_PROPERTIES_1, false));
 			this.add(Box.createRigidArea(GAP_DIMENSION));
			this.add(stages[STAGE_OBJECTS] = new Stage(this, "Objets", Icons.ICON_DISTRIBUTION_0, Icons.ICON_DISTRIBUTION_1, false));
			this.add(Box.createRigidArea(GAP_DIMENSION));
			this.add(stages[STAGE_GENERATION] = new Stage(this, "Generation", Icons.ICON_GENERATION_0, Icons.ICON_GENERATION_1, true));
			this.add(Box.createRigidArea(GAP_DIMENSION));
			this.add(stages[STAGE_NOISES] = new Stage(this, "Bruits", Icons.ICON_NOISIFICATION_0, Icons.ICON_NOISIFICATION_1, true));
			this.add(Box.createRigidArea(GAP_DIMENSION));
			this.add(stages[STAGE_NORMALIZATION] = new Stage(this, "Image", Icons.ICON_NORMALIZATION_0, Icons.ICON_NORMALIZATION_1, false));
			this.add(Box.createRigidArea(GAP_DIMENSION));
			this.add(stages[STAGE_STUDIES] = new Stage(this, "Etudes", Icons.ICON_STUDIES_0, Icons.ICON_STUDIES_1, false));
			this.add(Box.createRigidArea(GAP_DIMENSION));
			this.add(new InfosZone(simulSpec));
			this.add(Box.createRigidArea(GAP_DIMENSION));
			/* etat actuel */
			//stages[STAGE_OBJECTS].setAuto(simulSpec.DISTRIBUTION_AUTOMATIC);
			stages[STAGE_GENERATION].setAuto(simulSpec.GENERATION_AUTOMATIC);
			stages[STAGE_NOISES].setAuto(simulSpec.NOISIFICATION_AUTOMATIC);
			setSimulationStage(STAGE_PROPERTIES);
 		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "ControlBar()", ex);}
 	}


	

	
 	private final void setSimulationStage(int iStage)
 	{
		try{
			for(int i=0; i<iStage; i++) stages[i].setState(STATE_DONE);
			stages[iStage].setState(STATE_CURR);
			for(int i=iStage+1; i<stages.length; i++) stages[i].setState(STATE_WAIT);
			iActualStage = iStage;
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "setSimulationState()", ex);}
 	}
	

	public final void actionPerformed(ActionEvent ae)
	{
		try{
			if(ae.getSource() == stages[STAGE_PROPERTIES]) simulSpec.getMainFrame().showPropertiesDialog();
			if(ae.getSource() == stages[STAGE_OBJECTS]) simulSpec.getMainFrame().showDistributionDialog(false);
			if(ae.getSource() == stages[STAGE_GENERATION]) {simulSpec.getMainFrame().showGenerationDialog(false);}
			if(ae.getSource() == stages[STAGE_NOISES]) {simulSpec.getMainFrame().showNoisesDialog(false);}
			if(ae.getSource() == stages[STAGE_NORMALIZATION]) {simulSpec.getMainFrame().showNormalizationDialog(false);}
			if(ae.getSource() == stages[STAGE_GENERATION].getAutoButton()) {simulSpec.GENERATION_AUTOMATIC = stages[STAGE_GENERATION].getAutoButton().isSelected();}
			if(ae.getSource() == stages[STAGE_NOISES].getAutoButton()) {simulSpec.NOISIFICATION_AUTOMATIC = stages[STAGE_NOISES].getAutoButton().isSelected();}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}
	
	
	public void onSimulationStateChanged(boolean blSimulating)
	{
		if(blSimulating == false){
			/* on regarde ou on en est */
			if(iActualStage == STAGE_STUDIES){
				setSimulationStage(STAGE_OBJECTS);
				simulSpec.getMainFrame().showDistributionDialog(true);
				return;
			}
			/* arrete apres la distribution (generation) */
			if(iActualStage == STAGE_OBJECTS){
				setSimulationStage(STAGE_GENERATION);
				simulSpec.getMainFrame().showGenerationDialog(true);
				return;
			}
			/* arrete apres la generation (bruits) */
			if(iActualStage == STAGE_GENERATION){
				setSimulationStage(STAGE_NOISES);
				simulSpec.getMainFrame().showNoisesDialog(true);
				return;
			}
			/* arrete a la fin */
			if(iActualStage == STAGE_NORMALIZATION){
				setSimulationStage(STAGE_STUDIES);
				return;
			}
		}
	}
	
	public void onSimulationComponentsListChanged() {}
	public void onSimulationGeneratedComponentsChanged(int iGeneratedNumber) {}
	public void onSimulationStartDistribution() {setSimulationStage(STAGE_OBJECTS);}
	public void onSimulationStartGeneration() {setSimulationStage(STAGE_GENERATION);}
	public void onSimulationStartNoisation() {setSimulationStage(STAGE_NOISES);}
	public void onSimulationStartNormalization() {setSimulationStage(STAGE_NORMALIZATION);}

}



