/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package ui.simulation.editor.controlbar;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.Timer;

import kernel.Kernel;
import simulation.SimulationSpec;
import simulation._SimulationSpecListener;
import ui.UI;

public class InfosZone extends JPanel implements ActionListener, _SimulationSpecListener
{

	
	private static final long serialVersionUID			= 1L;
	
	private SimulationSpec simulSpec					= null;
	
	private JLabel lblInfo1								= null;
	private JLabel lblInfo2								= null;
	private JLabel lblInfo3								= null;

	private JProgressBar progressBar					= null;
	private Timer timer									= null;
	
	private long start									= 0;
	private int done									= 0;

	

	
	
	
 	public InfosZone(final SimulationSpec simulSpec)
 	{
 		super(null);
 		try{
 			this.setOpaque(false);
 			this.setMaximumSize(ControlBar.INFOS_DIMENSION);
 			this.setMinimumSize(ControlBar.INFOS_DIMENSION);
 			this.setPreferredSize(ControlBar.INFOS_DIMENSION);
 			/* icone des informations */
 			JLabel lblIconT = new JLabel("  ");
 			lblIconT.setBounds(0, 0, 12, 12);
 			/* labels des informations */
 			lblInfo1 = UI.createLabel(">nouvelle simulation<", UI.FONT_BOLD_SMALL);
 			lblInfo1.setBounds(14, -2, ControlBar.INFOS_DIMENSION.width-14, 14);
 			lblInfo2 = UI.createLabel("", UI.FONT_PLAIN_VERY_SMALL);
 			lblInfo2.setBounds(14, 9, ControlBar.INFOS_DIMENSION.width-14, 12);
 			lblInfo3 = UI.createLabel("", UI.FONT_PLAIN_VERY_SMALL);
 			lblInfo3.setBounds(14, 18, ControlBar.INFOS_DIMENSION.width-14, 12);
 			/* icone de la progression */
 			JLabel lblIconP = new JLabel("  ");
 			lblIconP.setBounds(0, ControlBar.INFOS_DIMENSION.height-14, 12, 12);
 			/* progression */
			progressBar = new JProgressBar(0, 1);
			progressBar.setStringPainted(false);
			progressBar.setString(null);
			progressBar.setValue(0);
			progressBar.setBounds(14, ControlBar.INFOS_DIMENSION.height-14, ControlBar.INFOS_DIMENSION.width-14, 14);
	 		/* on ajoute le tout */
			this.add(lblIconT);
	 		this.add(lblInfo1);
	 		this.add(lblInfo2);
	 		this.add(lblInfo3);
	 		this.add(lblIconP);
	 		this.add(progressBar);
	 		/* ecoute les infos de la simulation */
	 		this.simulSpec = simulSpec;
	 		this.simulSpec.addListener(this);
 		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "InfosZone()", ex);}
 	}



 	
 	
 	
 	
 	
	private final void addInfo(final String strInfo)
	{
		try{
			lblInfo3.setText(lblInfo2.getText());
			lblInfo2.setText(lblInfo1.getText().substring(1, lblInfo1.getText().length()-1));
			lblInfo1.setText(">"+strInfo+"<");
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "addInfo()", ex);}
	}
	
	
	
	
	
	
	
	
	
	
	private final void startProgress()
	{
		try{
			/* initialise la barre de progression */
			progressBar.setIndeterminate(true);
			progressBar.setStringPainted(false);
			/* pour calculer le temps */
			start = System.currentTimeMillis();
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "startProgress()", ex);}
	}
	
	private final void stopProgress()
	{
		try{
			/* arrete le compteur */
			if(timer != null){
				timer.stop();
				timer = null;
			}
			/* regarde ou on s'est arreté dans la simulation */
			if(simulSpec.getMatrix().getImage() == null){
				addInfo("initialisation de l'image");
				addInfo("attente de génération");
				progressBar.setValue(0);
				progressBar.setIndeterminate(false);
				progressBar.setStringPainted(false);
			}else{
				addInfo("image générée en " + ( (int) ( (System.currentTimeMillis() - start + 500) / 1000 ) ) + " sec");
				progressBar.setValue(progressBar.getMaximum());
				progressBar.setIndeterminate(false);
				progressBar.setStringPainted(true);
			}
			start = 0;
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "stopProgress()", ex);}
	}
	
	
	
	

	
	public final void actionPerformed(ActionEvent ae) {progressBar.setValue(done);}

	public void onSimulationComponentsListChanged() {}
	
	public void onSimulationGeneratedComponentsChanged(int iGeneratedNumber)
	{
		this.done = iGeneratedNumber;
	}
	
	public void onSimulationStateChanged(boolean blSimulating)
	{
		if(blSimulating == true){
			startProgress();
		}else{
			stopProgress();
		}
	}
	
	public void onSimulationStartDistribution()
	{
		addInfo("distribution des objets...");
	}
	
	public void onSimulationStartGeneration()
	{
		addInfo("génération des objets...");
		progressBar.setValue(0);
		progressBar.setMaximum(simulSpec.getComponents().size());
		progressBar.setString(null);
		progressBar.setStringPainted(true);
		progressBar.setIndeterminate(false);
		timer = new Timer(1000, this);
        timer.start();
        
	}
	
	public void onSimulationStartNoisation()
	{
		/* arrete le compteur */
		if(timer != null){
			timer.stop();
			timer = null;
		}
		addInfo("ajout des bruits...");
		progressBar.setStringPainted(false);
		progressBar.setIndeterminate(true);
	}
	
	public void onSimulationStartNormalization()
	{
		addInfo("normalisation de l'image...");
	}
	
	
	
}




