/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package ui.simulation.editor.controlbar;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

import kernel.Kernel;
import ui.UI;

public class Stage extends JButton implements MouseListener
{

	private static final long serialVersionUID			= 1L;
	
	private JCheckBox chkAuto							= null;
	
	private boolean blOver								= false;
	private boolean blCurrent							= false;
	
	private ImageIcon ic1								= null;
	private ImageIcon ic2								= null;
	
	
	
	
	
	
	
 	public Stage(final ActionListener lst, final String strTitle, final ImageIcon ic1, final ImageIcon ic2, final boolean blAuto)
 	{
 		super("");
 		try{
 			/* memorise les images */
 			this.ic1 = ic1;
 			this.ic2 = ic2;
 			/* proprietes */
			this.setToolTipText(strTitle);
			this.setPreferredSize(ControlBar.STAGE_DIMENSION);
			this.setMaximumSize(ControlBar.STAGE_DIMENSION);
			this.setMinimumSize(ControlBar.STAGE_DIMENSION);
			this.setOpaque(true);
			this.setBackground(Color.lightGray);
			this.setIcon(ic2);
			this.addActionListener(lst);
 			this.addMouseListener(this);
 			this.setLayout(null);
 			this.setBorder(null);
 			this.setMargin(new Insets(0, 0, 0, 0));
 			/* titre */
 			JLabel lbl = UI.createLabel(strTitle, UI.FONT_BOLD_ITALIC);
 			lbl.setHorizontalAlignment(SwingConstants.RIGHT);
 			lbl.setBounds(2, 0, ControlBar.STAGE_DIMENSION.width-6, 20);
 			this.add(lbl);
 			/* automatique */
 			if(blAuto == true){
 	 			chkAuto = UI.createCheckBox("", lst, false, UI.FONT_ITALIC);
 	 			chkAuto.setOpaque(false);
 	 			chkAuto.setBounds(ControlBar.STAGE_DIMENSION.width-21, ControlBar.STAGE_DIMENSION.height-20, 20, 20);
 	 			this.add(chkAuto);
 			}
 		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "Stage()", ex);}
 	}

 	
 	
 	
 	
 	public final void paint(Graphics g)
 	{
 		super.paint(g);
 		if(blCurrent || blOver){
 			g.setColor(Color.black);
 			g.drawRect(0, 0, ControlBar.STAGE_DIMENSION.width-1, ControlBar.STAGE_DIMENSION.height-1);
 	 		g.drawRect(0, 0, ControlBar.STAGE_DIMENSION.width-2, ControlBar.STAGE_DIMENSION.height-2);
 		}
 	}
 	
 	
 	

 	
 	public final void setState(final int iState)
 	{
 		switch(iState){
 			case ControlBar.STATE_DONE:
 				blCurrent = false;
 				this.setIcon(ic1);
 				break;
 			case ControlBar.STATE_CURR:
 				blCurrent = true;
 				this.setIcon(ic1);
 				break;
 			case ControlBar.STATE_WAIT:
 				blCurrent = false;
 				this.setIcon(ic2);
 				break;
 		}
 		this.repaint();
 	}
 	
 	
 	
 	
 	public final boolean isAuto() {if(chkAuto == null) return true; else return chkAuto.isSelected();}
 	public final void setAuto(boolean newVal) {if(chkAuto != null) chkAuto.setSelected(newVal);}
 	public final JCheckBox getAutoButton() {return chkAuto;}
 	

 	
 	public final void mouseClicked(MouseEvent e) {}
    public final void mousePressed(MouseEvent e) {}
    public final void mouseReleased(MouseEvent e) {}
    public final void mouseEntered(MouseEvent e) {blOver = true;}
    public final void mouseExited(MouseEvent e) {blOver = false;}

}



