/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package ui.simulation.editor.imagecanvas;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import kernel.Kernel;
import simulation.SimulationSpec;
import simulation._SimulationSpecListener;
 
public class ImageCanvas extends JPanel implements _SimulationSpecListener
{

	private static final long serialVersionUID		= 1L;
	
 	/* les couleurs */
 	public static final Color clBlank				= new Color(0, 0, 255);
 	public static final Color clText				= new Color(150, 150, 150);
 	public static final Color clPositions			= new Color(255, 0, 0);
 	public static final Color clBackground			= new Color(180, 180, 180);
 	public static final Color clShadow				= new Color(100, 100, 100);
 	public static final Color clInfo				= new Color(20, 20, 20);
 	public static final Color clInfoText			= new Color(200, 200, 200);
 	
 	/* la police */
 	public static Font ftText						= null;
 	

 	
 	
 	
	/* specifications de la simulation */
	private SimulationSpec spec						= null;

	/* zone image */
	private ImageCanvasMain main					= null;
	
	/* zone infos */
	private ImageCanvasInfos infos					= null;
	

	
	
	
	

 	


 	
 	
 	
 	public ImageCanvas(final SimulationSpec spec)
 	{
 		super(new BorderLayout());
 		try{
 			this.spec = spec;
 			spec.addListener(this);
	 		/* police */
	 		ftText = new Font(this.getFont().getName(), Font.PLAIN, 11);
 			/* initialise les composants */
 			infos = new ImageCanvasInfos(spec);
 			main = new ImageCanvasMain(spec, infos);
 			/* place l'image dans un scroll */
 			JScrollPane scroll = new JScrollPane(main);
 			scroll.setBorder(null);
 			scroll.getViewport().setBorder(null);
 			/* ajoute le tout */
 			this.add(scroll, BorderLayout.CENTER);
 			this.add(infos, BorderLayout.SOUTH);
 		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "ImageCanvas()", ex);}
 	}




 	
 	
 	
	public void onSimulationComponentsListChanged() {}
	public void onSimulationGeneratedComponentsChanged(int iGeneratedNumber) {}
	
	public void onSimulationStateChanged(boolean blSimulating)
	{
		try{
			if(blSimulating == true){
				main.setImage(null, spec.PARAM_IMAGE_SIZE_X, spec.PARAM_IMAGE_SIZE_Y);
			}else{
				infos.simulationSpecUpdated();
				main.setImage(spec.getMatrix().getImage(), spec.PARAM_IMAGE_SIZE_X, spec.PARAM_IMAGE_SIZE_Y);
			}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "onSimulationStateChanged()", ex);}
	}

	public void onSimulationStartDistribution() {}
	public void onSimulationStartGeneration() {}
	public void onSimulationStartNoisation() {}
	public void onSimulationStartNormalization() {}

 	
	

	
	

	public final int getMesureWidthX() {return main.getMesureWidthX();}
	public final void setMesureWidthX(ActionListener mesActionX, int newVal) {main.setMesureWidthX(mesActionX, newVal);}

	
	public final ImageCanvasMain getMainCanvas() {return main;}
	
	
}
 