/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package ui.simulation.editor.imagecanvas;

import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JPanel;

import simulation.SimulationSpec;

import kernel.Kernel;
import kernel.Maths;

public class ImageCanvasInfos extends JPanel
{

	private static final long serialVersionUID		= 1L;
	
	/* specifications de la simulation */
	private SimulationSpec simulSpec				= null;
	
	/* tailles et positions */
 	private int hInfSize							= 25;
 	private int iNextInfos1							= 70;
 	private int iNextInfos2							= 300;
	
 	/* zone d'infos */
 	private int cx									= 0;
 	private int cy									= 0;
 	
 	/* les chaines d'informations */
 	private String strDesc1							= "";
 	private String str30PixUnit						= "";
 	private String strDataColor						= "";
 	private String strDataTotal						= "";
 	private String strDataBack						= "";
 	private String strDataObjects					= "";
 	private String strDataNoise						= "";
 	

 	
 	
 	
 	
 	

 	public ImageCanvasInfos(final SimulationSpec simulSpec)
 	{
 		try{
 			this.simulSpec = simulSpec;
	 		/* autres proprietes */
	 		this.setOpaque(true);
	 		this.setBackground(ImageCanvas.clInfo);
	 		/* description */
	 		this.simulationSpecUpdated();
 		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "ImageCanvasInfos()", ex);}
 	}
 	
 	

	public final void updatePointer(final int cx, final int cy)
 	{
		this.cx = cx;
		this.cy = cy;
		if((cx > -1) && (cy > -1) && (cx < simulSpec.PARAM_IMAGE_SIZE_X) && (cy < simulSpec.PARAM_IMAGE_SIZE_Y)){
			strDataColor = "couleur : " + simulSpec.getMatrix().getNormalizedValue(cx, cy);
			strDataTotal = "total elec : " + Maths.toString(simulSpec.getMatrix().getTotalElectrons(cx, cy), 3);
			//strDataBack = "fond : " + Maths.toString(simulSpec.getMatrix().getBackNoiseIntensityMONO(), 3);
			strDataObjects = "objets : " + Maths.toString(simulSpec.getMatrix().getObjectsElectrons(cx, cy), 3);
			strDataNoise = "bruit : " + Maths.toString(simulSpec.getMatrix().getNoisesElectrons(cx, cy), 3);
		}else{
			strDataColor = strDataTotal = strDataBack = strDataObjects = strDataNoise = "";
		}
		this.repaint();
 	}
	
	public final void simulationSpecUpdated()
 	{
		this.strDesc1 = simulSpec.getFilterMono().getDescription();
		this.str30PixUnit = "" + (simulSpec.PARAM_PIXEL_RESOLUTION * 20.0) + " arcsec" + "  (x" + Maths.toString(simulSpec.DISPLAY_ZOOM_VALUE, 3) + ")";
		this.repaint();
 	}

 	public final Dimension getMinimumSize() {return new Dimension(hInfSize, hInfSize);}
 	public final Dimension getMaximumSize() {return new Dimension(hInfSize, hInfSize);}
 	public final Dimension getPreferredSize() {return new Dimension(hInfSize, hInfSize);}
 	
	public final void paint(Graphics g)
 	{
		try{
	 		super.paint(g);
	 		g.setFont(ImageCanvas.ftText);
			g.setColor(ImageCanvas.clInfoText);
			g.drawLine(iNextInfos1, 6+11, 20+iNextInfos1, 6+11);
			g.drawLine(iNextInfos1, 3+11, iNextInfos1, 9+11);
			g.drawLine(20+iNextInfos1, 3+11, 20+iNextInfos1, 9+11);
			g.drawString("X = " + cx, 1, 10);
			g.drawString("Y = " + cy, 1, 23);
			g.drawString(strDesc1, iNextInfos1-2, 10);
			g.drawString(str30PixUnit, 25+iNextInfos1, 23);
			g.drawString(strDataTotal, iNextInfos2, 10);
			g.drawString(strDataColor, iNextInfos2+220, 10);
			g.drawString(strDataBack, iNextInfos2, 23);
			g.drawString(strDataObjects, iNextInfos2+110, 23);
			g.drawString(strDataNoise, iNextInfos2+220, 23);
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "paint()", ex);}
	 	}
		
		
	 }

 