/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package ui.simulation.editor.imagecanvas;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;

import kernel.Kernel;
import ressources.Icons;
import simulation.SimulationSpec;
import simulation.components.ImageComponent;
import ui.UI;

public class ImageCanvasMain extends JPanel implements ActionListener, MouseMotionListener, MouseListener
 {

	private static final long serialVersionUID		= 1L;

	
	/* specifications de la simulation */
	private SimulationSpec simulSpec				= null;
 	
	/*  */
	private ImageCanvasInfos infos					= null;
	
	/* l'image */
 	private Image img								= null;
 	
 	/* les dimensions reelles de l'image */
 	private int iW									= 0;
 	private int iH									= 0;
 	
 	/* la position de l'image dans la zone */
 	private int dx									= 0;
 	private int dy									= 0;
 	
 	/* la taille modifiee */
 	private int w									= 0;
 	private int h									= 0;
 	
 	/* zone d'infos */
 	private int cx									= 0;
 	private int cy									= 0;
 	private int cxZoom								= 0;
 	private int cyZoom								= 0;
 	
 	/* mesure en x */
 	private boolean mesMovX							= false;
 	private int mesPosX								= 0;
 	private int mesPosZoomX							= 0;
 	private int mesWidthX							= 0;
 	private int mesWidthZoomX						= 0;
 	private ActionListener mesActionX				= null;

 	
 	JPanel pnlControl		= null;
 	
 	
 	JButton btnPlus = null;
 	JButton btnMinus = null;
 	JButton btnDraw = null;
 	
 	

 	

 	public ImageCanvasMain(final SimulationSpec simulSpec, final ImageCanvasInfos infos)
 	{
 		super(null);
 		try{
 			this.simulSpec = simulSpec;
 			this.infos = infos;
	 		/* taille de l'image */
	 		this.updateImageAndRepaint(null, simulSpec.PARAM_IMAGE_SIZE_X, simulSpec.PARAM_IMAGE_SIZE_Y);
	 		/* autres proprietes */
	 		this.setOpaque(true);
	 		this.setBackground(ImageCanvas.clBackground);
     		/* evenements */
     		this.addMouseListener(this);
     		this.addMouseMotionListener(this);
     		
     		buildControlPanel();
     		
     		
     		
     		
     		this.add(pnlControl);
     
     		
     		
 		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "ImageCanvas()", ex);}
 	}


 	private void buildControlPanel()
 	{
 		try{
 			
	 		/* on construit les elements graphiques */
	 		pnlControl = UI.createPanel(null);
			pnlControl.setLayout(new BoxLayout(pnlControl, BoxLayout.Y_AXIS));

			//pnlControl.setBackground(Color.red);
			pnlControl.setOpaque(false);

			pnlControl.add(btnPlus = UI.createIcon("Agrandir l'image", this, Icons.ICON_ZOOM_PLUS_0, Icons.ICON_ZOOM_PLUS_1));
			pnlControl.add(btnMinus = UI.createIcon("Réduire l'image", this, Icons.ICON_ZOOM_MINUS_0, Icons.ICON_ZOOM_MINUS_1));
			pnlControl.add(Box.createRigidArea(new Dimension(5, 5)));
			pnlControl.add(btnDraw = UI.createIcon("Dessiner les positions des objets", this, Icons.ICON_POSITIONS_OFF_0, Icons.ICON_POSITIONS_OFF_1));

			btnPlus.addMouseListener(this);
			btnMinus.addMouseListener(this);
			btnDraw.addMouseListener(this);
			
			pnlControl.addMouseListener(this);
			pnlControl.setBounds(0, 0, 28, 89);
 		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "buildControlPanel()", ex);}
 	}

 	

 	
 	

	
 	
 	
 	public final void paint(Graphics g)
 	{
		try{
			super.paint(g);
	 		this.updatePositionAndSizeDesign();
			/* dessine l'image */
			if(img != null){
				g.drawImage(img, dx, dy, w, h, null);
			}else{
				g.setColor(ImageCanvas.clBlank);
	 			g.fillRect(dx, dy, w, h);
	 		}
			/* ombre */
			g.setColor(ImageCanvas.clShadow);
			g.fillRect(dx+w, dy+5, 5, h);
			g.fillRect(dx+5, dy+h, w, 5);
	 		/* echelle */
			g.setColor(ImageCanvas.clText);			
			g.drawLine(3+dx, 6+dy, 3+dx+(int)(20*simulSpec.DISPLAY_ZOOM_VALUE), 6+dy);
			g.drawLine(3+dx, 3+dy, 3+dx, 9+dy);
			g.drawLine(3+dx+(int)(20*simulSpec.DISPLAY_ZOOM_VALUE), 3+dy, 3+dx+(int)(20*simulSpec.DISPLAY_ZOOM_VALUE), 9+dy);
			/* positions */
			if(simulSpec.DISPLAY_OBJECTS_POSITIONS) drawPositions(g);
			/* mesures */
			if(mesWidthX > 0){
				g.setColor(ImageCanvas.clBlank);
				g.drawLine(dx, mesPosZoomX, dx+w, mesPosZoomX);
				g.drawLine(dx, mesPosZoomX+mesWidthZoomX, dx+w, mesPosZoomX+mesWidthZoomX);
			}
			/* pointer */
			if(simulSpec.DISPLAY_ZOOM_VALUE > 1.0){
				g.setColor(ImageCanvas.clPositions);
				g.drawRect(cxZoom, cyZoom, (int) simulSpec.DISPLAY_ZOOM_VALUE, (int) simulSpec.DISPLAY_ZOOM_VALUE);
			}
			
			if(pnlControl.isVisible()) pnlControl.repaint();
			
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "paint()", ex);}
 	}
 	
	private final void drawPositions(Graphics g)
	{
		try{
			ImageComponent cpn;
			g.setColor(ImageCanvas.clPositions);
			int x, y;
			
			for(int i=0; i<simulSpec.getComponents().size(); i++){
				cpn = (ImageComponent)(simulSpec.getComponents().get(i));
				x = ((int)(cpn.getPosX() * simulSpec.DISPLAY_ZOOM_VALUE)) + dx;
				y = ((int)(cpn.getPosY() * simulSpec.DISPLAY_ZOOM_VALUE)) + dy;
				g.drawLine(x, y-5, x, y+5);
				g.drawLine(x-5, y, x+5, y);
			}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "startOrStopSimulation()", ex);}
	}
 	
	
	
	

 	private final void updateImageAndRepaint(Image img, final int sizeX, final int sizeY)
 	{
 		this.img = img;
 		this.iW = sizeX;
 		this.iH = sizeY;
 		this.updatePositionAndSizeDesign();
 		this.repaint();
 	}
 	
 	public final void updateZoomAndRepaint()
 	{
 		this.updatePositionAndSizeDesign();
 		this.updateUI();
 	}
 
 	private final void updateCursorPositionAndRepaint(final int x, final int y)
 	{
 		this.cx = (int) ( (x - dx) / simulSpec.DISPLAY_ZOOM_VALUE );
 		this.cy = (int) ( (y - dy) / simulSpec.DISPLAY_ZOOM_VALUE );
 		this.updateCursorAndSizeDesign();
 		this.repaint();
 	}
 
	public final void updateMeasureAndRepaint(ActionListener mesActionX, int newVal)
	{
		if(newVal > iH) newVal = iH;
		this.mesActionX = mesActionX;
		if((newVal > 0) && (mesActionX != null)){
			this.mesMovX = true;
			this.mesWidthX = newVal;
			this.updateMeasurePositionDesign();
		}else{
			this.mesMovX = false;
			this.mesWidthX = 0;
		}
		this.repaint();
	}
 	
 	
	
	
	
	
	
	
	
	
	
	public final void setZoomFactor(final double newVal)
	{
		try{
			simulSpec.DISPLAY_ZOOM_VALUE = newVal;
			if(simulSpec.DISPLAY_ZOOM_VALUE < 0.2) simulSpec.DISPLAY_ZOOM_VALUE = 0.2;
			updateZoomAndRepaint();
			infos.simulationSpecUpdated();
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "setZoomFactor()", ex);}
	}
	
	public void setDrawPoints(final boolean newVal)
	{
		try{
			simulSpec.DISPLAY_OBJECTS_POSITIONS = newVal;
			if(newVal == false){
				btnDraw.setIcon(Icons.ICON_POSITIONS_OFF_0);
				btnDraw.setRolloverIcon(Icons.ICON_POSITIONS_OFF_1);
			}else{
				btnDraw.setIcon(Icons.ICON_POSITIONS_ON_0);
				btnDraw.setRolloverIcon(Icons.ICON_POSITIONS_ON_1);
			}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "setDrawPoints()", ex);}
	}
	
 	
 	
	
	

	
	
	
	
	

 	private final void updatePositionAndSizeDesign()
 	{
 		this.w = (int) ((double)iW * simulSpec.DISPLAY_ZOOM_VALUE);
 		this.h = (int) ((double)iH * simulSpec.DISPLAY_ZOOM_VALUE);
 		this.dx = ((this.getWidth() - w) / 2);
 		this.dy = ((this.getHeight() - h) / 4);
 		if(mesWidthX > 0) this.updateMeasurePositionDesign();
 		this.updateCursorAndSizeDesign();
 	}
 	
 	private final void updateCursorAndSizeDesign()
 	{
 		/*
 		int newPos;
 		newPos = dx + (int) (cx * dblZoom);
 		if(newPos < dx) cxZoom = dx;
		else{
			if(newPos >= dx+w) cxZoom = dx+w-(int)dblZoom;
			else cxZoom = newPos;
		}
 		newPos = dy + (int) (cy * dblZoom);
 		if(newPos < dy) cyZoom = dy;
		else{
			if(newPos >= dy+h) cyZoom = dy+h-(int)dblZoom;
			else cyZoom = newPos;
		}
		*/
 		cxZoom = (int) (cx * simulSpec.DISPLAY_ZOOM_VALUE) + dx;
 		cyZoom = (int) (cy * simulSpec.DISPLAY_ZOOM_VALUE) + dy;
 		/* deplace la mesure */
 		if(mesMovX == true) this.updateMeasurePositionDesign();
 		/* on annonce que le curseur a change de place */
 		infos.updatePointer(this.cx, this.cy);
 	}

	public final void updateMeasurePositionDesign()
	{
		this.mesWidthZoomX = (int) ((double)mesWidthX * simulSpec.DISPLAY_ZOOM_VALUE);
		if(mesMovX == true){
			if(cyZoom < dy) mesPosZoomX = dy;
			else{
				if(cyZoom >= dy+h-mesWidthZoomX) mesPosZoomX = dy+h-mesWidthZoomX;
				else mesPosZoomX = cyZoom;
			}
			/*
			if(mesPosZoomX+mesWidthZoomX > dy+h) mesPosZoomX = dy+h-mesWidthZoomX;
			else mesPosZoomX = cyZoom;
			*/
		}else{
			mesPosZoomX = (int) (mesPosX * simulSpec.DISPLAY_ZOOM_VALUE) + dy;
		}
	}

 	
 	
 	
 	
 	
 	

 	public final Dimension getMinimumSize() {return new Dimension(w+20, h+10);}
 	public final Dimension getMaximumSize() {return new Dimension(w+20, h+10);}
 	public final Dimension getPreferredSize() {return new Dimension(w+20, h+10);}
	
	public final Image getImage() {return img;}
	public final void setImage(Image newVal, final int sizeX, final int sizeY) {this.updateImageAndRepaint(newVal, sizeX, sizeY);}
	
	public final int getImagePosX() {return dx;}
	public final int getImagePosY() {return dy;}

	public final int getMesureWidthX() {return mesWidthX;}
	public final void setMesureWidthX(ActionListener mesActionX, int newVal) {updateMeasureAndRepaint(mesActionX, newVal);}

    public void mouseDragged(MouseEvent e) {;}
    public void mouseMoved(MouseEvent e) {updateCursorPositionAndRepaint(e.getX(), e.getY());}

    public void mousePressed(MouseEvent e) {;}
    public void mouseReleased(MouseEvent e) {;}
    public void mouseEntered(MouseEvent e) {pnlControl.setVisible(true);}
    public void mouseExited(MouseEvent e) {pnlControl.setVisible(false);}
    public void mouseClicked(MouseEvent e)
    {
    	if(mesMovX == true){
        	mesMovX = false;
        	mesPosX = cy;
        	if(mesActionX != null) mesActionX.actionPerformed(new ActionEvent(this, 1, ""+mesPosX)); /* 1=X, 2=Y */
    	}
    }

  
    
	public final void actionPerformed(ActionEvent ae)
	{
		try{
			if(ae.getSource() == btnPlus) {setZoomFactor(simulSpec.DISPLAY_ZOOM_VALUE + 0.2);}
			if(ae.getSource() == btnMinus) {setZoomFactor(simulSpec.DISPLAY_ZOOM_VALUE - 0.2);}
			if(ae.getSource() == btnDraw) {setDrawPoints(!simulSpec.DISPLAY_OBJECTS_POSITIONS);}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}
	
 	
 	
 	
	


	
	
	
	
	
	


 }
 


