/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package ui.simulation.frames;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;

import kernel.Generator;
import kernel.Kernel;
import ressources.Lang;
import simulation.SimulationSpec;
import simulation._SimulationSpecListener;
import ui.UI;
import ui.common.DialogFrame;

public class GenerationDialog extends DialogFrame implements ActionListener, _SimulationSpecListener
{

	private static final long serialVersionUID 			= 1L;

	private SimulationSpec simulSpec					= null;

	private JButton btnUpdate							= null;
	
	private JTextField txtMaxPixels						= null;
	

	
	
	
	
	
	
	
	
	public GenerationDialog(final SimulationSpec simulSpec)
	{
		super(simulSpec.getMainFrame(), Lang.STR_TITLE_GENERATE_OBJECTS, false);
		try{
			this.simulSpec = simulSpec;
			/* construit le contenu */
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 0, 3), 0, 0);
			JPanel pnlTmpText = UI.createPanel(new GridBagLayout());
			pnlTmpText.setBorder(new LineBorder(Color.darkGray, 1));
			/* optimisation */
			c.gridwidth = 2;
			pnlTmpText.add(UI.createLabel("Optimisation", UI.FONT_BOLD), c);
			c.gridwidth = 1;
			/* rayon de calcul */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("  " + "Rayon max de calcul", UI.FONT_ITALIC), c);
			c.gridx = 1;
			pnlTmpText.add(txtMaxPixels = new JTextField(""+simulSpec.GENERATION_OBJECTS_SIZE), c);
			/* texte d'information */
			c.gridy++;
			c.gridx = 0;
			c.gridwidth = 2;
			pnlTmpText.add(UI.createLabel("Taille de la zone dans laquelle seront dessinés les objets.", UI.FONT_PLAIN_SMALL), c);
			c.gridy++;
			pnlTmpText.add(UI.createLabel("Mettre 0 pour spécifier toute l'image.", UI.FONT_PLAIN_SMALL), c);
			c.gridwidth = 1;
			/* mise a jour */
			c.gridy++;
			c.gridx = 1; 
			c.insets = new Insets(3, 3, 3, 3);
			pnlTmpText.add(btnUpdate = UI.createButton("Mettre à jour", this, null), c);
			btnUpdate.setEnabled(false);
			/* ajoute le tout */
			JPanel pnl = UI.createPanel(new BorderLayout());
			pnl.add(UI.createLabel(Lang.STR_TITLE_GENERATE_OBJECTS, UI.FONT_BOLD_BIG), BorderLayout.NORTH);
			pnl.add(pnlTmpText, BorderLayout.CENTER);
			setContent(pnl, BUTTONS_CLOSE);
			setSize(340, this.getHeight());
			/* on demande l'ecoute de la simulation */
			this.simulSpec.addListener(this);
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "GenerationDialog()", ex);}
	}


	
	

	
	
	private final void startGeneration()
	{
		try{
			/* on modifie l'optimisation */
			try{
				int opt = Integer.parseInt(txtMaxPixels.getText());
				if(opt < 0) opt = 0;
				simulSpec.GENERATION_OBJECTS_SIZE = opt;
			}catch(Exception ex) {}
			txtMaxPixels.setText(""+simulSpec.GENERATION_OBJECTS_SIZE);
			/* on lance la simulation */
			Generator.generate(simulSpec, Generator.GENERATE_FROM_GENERATION);
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "startGeneration()", ex);}
	}


	

	
	
	public final void actionPerformed(ActionEvent ae)
	{
		super.actionPerformed(ae);
		try{
			if(ae.getSource() == btnUpdate) {startGeneration(); return;}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}

	
	
	
	
	public void onSimulationComponentsListChanged() {}
	public void onSimulationGeneratedComponentsChanged(int iGeneratedNumber) {}
	public void onSimulationStateChanged(boolean blSimulating) {btnUpdate.setEnabled(blSimulating?false:(simulSpec.getComponents().size()>0?true:false));}

	public void onSimulationStartDistribution() {}
	public void onSimulationStartGeneration() {}
	public void onSimulationStartNoisation() {}
	public void onSimulationStartNormalization() {}
	

	
}
