/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package ui.simulation.frames;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;

import kernel.Kernel;
import kernel.functions.FunctionsFactory;
import ressources.Lang;
import simulation.SimulationSpec;
import simulation._SimulationSpecListener;
import ui.UI;
import ui.common.DialogFrame;
import ui.common.GraphCanvas;

public class IntensityUtilDialog extends DialogFrame implements ActionListener, _SimulationSpecListener
{

	private static final long serialVersionUID 			= 1L;

	private SimulationSpec simulSpec					= null;

	private JTextField txtWidth							= null;

	private JTextField txtXMin							= null;
	private JTextField txtXMax							= null;
	private JTextField txtYMin							= null;
	private JTextField txtYMax							= null;
	
	private JCheckBox chkYAuto							= null;

	private JButton btnMeasure							= null;
	private JButton btnUpdate							= null;
	
	private GraphCanvas cvIntens						= null;
	
	private JRadioButton rdNormalizedData				= null;
	
	private int iLastStartX								= -1;

	
	

	public IntensityUtilDialog(final SimulationSpec simulSpec)
	{
		super(simulSpec.getMainFrame(), Lang.STR_TITLE_INTENSITY_STATS, false);
		try{
			this.simulSpec = simulSpec;
			/* construit le contenu */
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 0, 3), 0, 0);
			JPanel pnlTmpText = UI.createPanel(new GridBagLayout());
			pnlTmpText.setBorder(new LineBorder(Color.darkGray, 1));
			ButtonGroup g = new ButtonGroup();
			/* titre */
			pnlTmpText.add(UI.createLabel("Resultats", UI.FONT_BOLD), c);
			/* graphe */
			c.gridy++;
			c.gridx = 0;
			c.gridwidth = 3;
			cvIntens = new GraphCanvas(0, simulSpec.PARAM_IMAGE_SIZE_X-1, 50, 5);
			cvIntens.setPreferredSize(new Dimension(320, 240));
			pnlTmpText.add(cvIntens, c);
			c.gridwidth = 1;
			/* plage x */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("  " + "Plage abscisse", UI.FONT_ITALIC), c);
			c.gridx = 1;
			pnlTmpText.add(txtXMin = new JTextField("0"), c);
			c.gridx = 2;
			pnlTmpText.add(txtXMax = new JTextField(""+(simulSpec.PARAM_IMAGE_SIZE_X-1)), c);
			/* plage y */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("  " + "Plage ordonnees", UI.FONT_ITALIC), c);
			c.gridx = 1;
			c.gridwidth = 2;
			pnlTmpText.add(chkYAuto = UI.createCheckBox("automatique", this, true, UI.FONT_ITALIC), c);
			c.gridwidth = 1;
			c.gridy++;
			c.gridx = 1;
			pnlTmpText.add(txtYMin = new JTextField("-10"), c);
			c.gridx = 2;
			pnlTmpText.add(txtYMax = new JTextField("255"), c);
			/* type de donnees */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("  " + "Type de donnees", UI.FONT_ITALIC), c);
			c.gridx = 1;
			c.gridwidth = 2;
			pnlTmpText.add(rdNormalizedData = UI.createRadioButton("valeurs normalisees", this, true, g, UI.FONT_ITALIC), c);
			c.gridy++;
			c.gridx = 1;
			pnlTmpText.add(UI.createRadioButton("nombres d'electrons", this, true, g, UI.FONT_ITALIC), c);
			/* mettre a jour */
			c.gridy++;
			c.gridx = 1;
			c.gridwidth = 2;
			pnlTmpText.add(btnUpdate = UI.createButton("Mettre a jour", this, UI.SIZE_BUTTON), c);
			c.gridwidth = 1;
			/* titre */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("Nouvelle mesure", UI.FONT_BOLD), c);
			/* largeur */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("  " + "Largeur de la mesure", UI.FONT_ITALIC), c);
			c.gridx = 1;
			c.gridwidth = 2;
			pnlTmpText.add(txtWidth = new JTextField(""+simulSpec.UTILS_INTENSITY_MEASURE_WIDTH), c);
			c.gridwidth = 1;
			/* mesurer */
			c.gridy++;
			c.insets = new Insets(3, 3, 3, 3);
			c.gridx = 1;
			c.gridwidth = 2;
			pnlTmpText.add(btnMeasure = UI.createButton("Mesurer", this, UI.SIZE_BUTTON), c);
			c.gridwidth = 1;
			
			if(simulSpec.getMatrix().isNormalized() == false) btnMeasure.setEnabled(false);
			btnUpdate.setEnabled(false);
			toggleAutoY();
			
			/* ajoute le tout */
			JPanel pnl = UI.createPanel(new BorderLayout());
			pnl.add(UI.createLabel(Lang.STR_TITLE_INTENSITY_STATS, UI.FONT_BOLD_BIG), BorderLayout.NORTH);
			pnl.add(pnlTmpText, BorderLayout.CENTER);
			setContent(pnl, BUTTONS_CLOSE);
			setSize(340, this.getHeight());
			/* on demande l'ecoute de la simulation */
			this.simulSpec.addListener(this);

		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "PanelObjectsAdd()", ex);}
	}
	
	
	
	
	
	
	
	
	
	
	
	private final void doMeasure()
	{
		try{
			/* on enregistre les valeurs */
			try{simulSpec.UTILS_INTENSITY_MEASURE_WIDTH = Integer.parseInt(txtWidth.getText());}catch(Exception ex) {txtWidth.setText(""+simulSpec.UTILS_INTENSITY_MEASURE_WIDTH);}
			
			/* on demande une mesure */
			simulSpec.getMainFrame().editor().getImageCanvas().setMesureWidthX(this, simulSpec.UTILS_INTENSITY_MEASURE_WIDTH);
			
			btnMeasure.setEnabled(false);
			btnUpdate.setEnabled(false);
			txtWidth.setEnabled(false);
		
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "doMeasure()", ex);}
	}
	
	private final void calcMeasureX(final int startX)
	{
		try{
			if(startX < 0) return;
			/* nettoie les anciennes fonctions */
			cvIntens.clearFunctions();
			/* plage pour x */
			int iMinX = 0;
			int iMaxX = simulSpec.PARAM_IMAGE_SIZE_X-1;
			try{iMinX = Integer.parseInt(txtXMin.getText());}catch(Exception ex) {}
			try{iMaxX = Integer.parseInt(txtXMax.getText());}catch(Exception ex) {}
			if(iMinX > iMaxX) {iMinX = 0; iMaxX = simulSpec.PARAM_IMAGE_SIZE_X-1;}
			txtXMin.setText(""+iMinX);
			txtXMax.setText(""+iMaxX);
			cvIntens.setXPolicy(iMinX, iMaxX, 100, 10);
			/* plage pour y */
			double dblMinY = 0;
			double dblMaxY = 0;
			
			if(chkYAuto.isSelected() == true){
				cvIntens.setYPolicy();
			}else{
				dblMinY = -10.0;
				dblMaxY = 255.0 * (double) (simulSpec.UTILS_INTENSITY_MEASURE_WIDTH);
				try{dblMinY = Double.parseDouble(txtYMin.getText());}catch(Exception ex) {}
				try{dblMaxY = Double.parseDouble(txtYMax.getText());}catch(Exception ex) {}
				if(dblMinY > dblMaxY) {dblMinY = -10.0; dblMaxY = 255.0 * (double) (simulSpec.UTILS_INTENSITY_MEASURE_WIDTH);}
				txtYMin.setText(""+dblMinY);
				txtYMax.setText(""+dblMaxY);
				cvIntens.setYPolicy(dblMinY, dblMaxY, (dblMaxY-dblMinY)/5, (dblMaxY-dblMinY)/50);
			}
			
			
			
			
			/* on regarde le type de mesure */
			if(rdNormalizedData.isSelected() == true){
				
				/* donnees normalisees */
				double tot;
				String points = FunctionsFactory.POINTS_FUNCTION;
				/* on fait les calculs */
				for(int i=iMinX; i<=iMaxX; i++){
					tot = 0.0;
					if((i > -1) && (i < simulSpec.PARAM_IMAGE_SIZE_X)){
						for(int j=startX; (j<(startX+simulSpec.UTILS_INTENSITY_MEASURE_WIDTH))&&(j<simulSpec.PARAM_IMAGE_SIZE_Y); j++){
							tot += (double) simulSpec.getMatrix().getNormalizedValue(i, j);
						}
					}
					points = points + "[" + i + ";" + tot + "]";
				}
				/* ajoute les fonctions */
				cvIntens.addFunction(FunctionsFactory.parse(null, null, points, null));
				
			}else{
				
				/* donnees brutes */
				double tot, obj, brt, fnd;
				String ptot = FunctionsFactory.POINTS_FUNCTION, pobj = ptot, pbrt = ptot, pfnd = ptot;
				/* on fait les calculs */
				for(int i=iMinX; i<=iMaxX; i++){
					tot = obj = brt = fnd = 0.0;
					if((i > -1) && (i < simulSpec.PARAM_IMAGE_SIZE_X)){
						for(int j=startX; (j<(startX+simulSpec.UTILS_INTENSITY_MEASURE_WIDTH))&&(j<simulSpec.PARAM_IMAGE_SIZE_Y); j++){
							tot += simulSpec.getMatrix().getTotalElectrons(i, j);
							obj += simulSpec.getMatrix().getObjectsElectrons(i, j);
							brt += simulSpec.getMatrix().getNoisesElectrons(i, j);
							fnd += 0;
						}
					}
					ptot = ptot + "[" + i + ";" + tot + "]";
					pobj = pobj + "[" + i + ";" + obj + "]";
					pbrt = pbrt + "[" + i + ";" + brt + "]";
					pfnd = pfnd + "[" + i + ";" + fnd + "]";
				}
				/* ajoute les fonctions */
				cvIntens.addFunction(FunctionsFactory.parse(null, null, ptot, null));
				cvIntens.addFunction(FunctionsFactory.parse(null, null, pobj, null));
				cvIntens.addFunction(FunctionsFactory.parse(null, null, pbrt, null));
				cvIntens.addFunction(FunctionsFactory.parse(null, null, pfnd, null));
			}
			

			
			
			/* ajoute le zero */
			cvIntens.addFunction(FunctionsFactory.parse(null, null, "POINTS:[0;0]", Color.lightGray));
			
			
			/* memorise */
			iLastStartX = startX;	
			btnUpdate.setEnabled(true);
			btnMeasure.setEnabled(true);
			txtWidth.setEnabled(true);
			
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "calcMeasureX()", ex);}
	}
	

	
	
	
	
	private final void toggleAutoY()
	{
		try{
			txtYMin.setEnabled(!chkYAuto.isSelected());
			txtYMax.setEnabled(!chkYAuto.isSelected());
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "toggleAutoY()", ex);}
	}
	

	
	
	
	public final void actionPerformed(ActionEvent ae)
	{
		super.actionPerformed(ae);
		try{

			if(ae.getSource() == chkYAuto) {toggleAutoY(); return;}
			
			if(ae.getSource() == btnMeasure) {doMeasure(); return;}
			
			if(ae.getSource() == btnUpdate) {calcMeasureX(iLastStartX); return;}
			
			if(ae.getSource() == simulSpec.getMainFrame().editor().getImageCanvas().getMainCanvas()) {if(ae.getID() == 1) calcMeasureX(Integer.parseInt(ae.getActionCommand())); return;}

		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}

	
	public void onSimulationComponentsListChanged() {}
	public void onSimulationGeneratedComponentsChanged(int iGeneratedNumber) {}
	public void onSimulationStateChanged(boolean blSimulating) {btnMeasure.setEnabled((!blSimulating)&&(simulSpec.getMatrix().isNormalized())); btnUpdate.setEnabled((!blSimulating)&&(iLastStartX>-1)&&(simulSpec.getMatrix().isNormalized()));}

	public void onSimulationStartDistribution() {}
	public void onSimulationStartGeneration() {}
	public void onSimulationStartNoisation() {}
	public void onSimulationStartNormalization() {}
	
}
