/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package ui.simulation.frames;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.border.LineBorder;

import kernel.Generator;
import kernel.Kernel;
import kernel.functions._Function;
import ressources.Lang;
import simulation.SimulationSpec;
import simulation._SimulationSpecListener;
import ui.UI;
import ui.common.DialogFrame;
import ui.common.GraphCanvas;

public class NoisesDialog extends DialogFrame implements ActionListener, _SimulationSpecListener
{

	public static final String VAR_OBJECTS_ELECT		= "%objects_elec%";
	public static final String VAR_OBS_TIME				= "%obs_time%";
	public static final String VAR_LAMBDA				= "%lambda%";
	public static final String VAR_DLAMBDA				= "%dlambda%";
	public static final String VAR_TELESC_DIAM			= "%telesc_diam%";
	public static final String VAR_PIX_RES				= "%pix_res%";
	

	private static final long serialVersionUID 			= 1L;

	private SimulationSpec simulSpec					= null;

	private JButton btnAdd								= null;
	private JButton btnUpdate							= null;

	private JTextArea txtFormula						= null;
	
	private JPopupMenu popup							= null;
	
	
	
	
	
	
	
	
	
	public NoisesDialog(final SimulationSpec simulSpec)
	{
		super(simulSpec.getMainFrame(), Lang.STR_TITLE_NOISES, false);
		try{
			this.simulSpec = simulSpec;
			/* construit le contenu */
			JPanel pnlInside;
			JPanel pnlContent;
			JPanel pnlGridBag;
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 0, 3), 0, 0);			
			/* zone du haut */
			pnlContent = UI.createPanel(new BorderLayout());
			pnlInside = UI.createPanel(new BorderLayout());
			pnlInside.add(UI.createLabel("  " + "Valeur du bruit (elect)", UI.FONT_ITALIC), BorderLayout.CENTER);
			pnlInside.add(btnAdd = UI.createButton("+", this, UI.SIZE_SMALL_ICON), BorderLayout.EAST);
			btnAdd.setEnabled(false);
			pnlGridBag = UI.createPanel(new GridBagLayout());
			pnlGridBag.add(pnlInside, c);
			pnlContent.add(pnlGridBag, BorderLayout.NORTH);
			pnlGridBag = UI.createPanel(new GridBagLayout());
			pnlGridBag.add(UI.createScrollPane(txtFormula = UI.createTextArea(simulSpec.NOISES_FORMULA), null), c);
			pnlContent.add(pnlGridBag, BorderLayout.CENTER);
			/* zone a onglets */
			JTabbedPane tab = new JTabbedPane();
			tab.addTab("Script", pnlContent);
			/* zone principale */
			pnlContent = UI.createPanel(new BorderLayout());
			/* ajoute les onglets */
			pnlContent.add(tab, BorderLayout.CENTER);
			/* ajoute les boutons */
			c.insets = new Insets(3, 3, 3, 3);
			pnlGridBag = UI.createPanel(new GridBagLayout());
			pnlGridBag.add(UI.createPanel(null), c);
			c.weightx = 0.2; c.gridx = 1;
			pnlGridBag.add(btnUpdate = UI.createButton("Mettre à jour", this, null), c);
			btnUpdate.setEnabled(false);
			pnlContent.add(pnlGridBag, BorderLayout.SOUTH);
			/* ajoute le tout */
			JPanel pnl = UI.createPanel(new BorderLayout());
			pnl.add(UI.createLabel(" "+Lang.STR_TITLE_NOISES, UI.FONT_BOLD_BIG), BorderLayout.NORTH);
			pnl.add(pnlContent, BorderLayout.CENTER);
			pnlContent.setBorder(new LineBorder(Color.darkGray, 1));
			/* taille et autres */
			this.setContent(pnl, BUTTONS_CLOSE);
			this.setResizable(true);
			/* construit le menu */
			this.buildPopupMenu();
			/* on demande l'ecoute de la simulation */
			this.simulSpec.addListener(this);
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "NoisesDialog()", ex);}
	}


	private final void buildPopupMenu()
	{
		try{
			/* construit le menu */
			popup = new JPopupMenu();
			popup.add(UI.createLabel(" Ajouter...", UI.FONT_BOLD));
			popup.addSeparator();
			popup.add(UI.createLabel(" Objets ponctuels", UI.FONT_ITALIC));
			popup.add(UI.createMenuItem("[ x_range ; y_range ; number ; flux_range ]", this, null, "[ x_range ; y_range ; number ; flux_range ]"));
			popup.add(UI.createMenuItem("[ x ; y ; 1 ; 1e3 ]", this, null, "[ x ; y ; 1 ; 1e3 ]"));
			popup.add(UI.createMenuItem("[ 0..XMAX ; 0..YMAX ; 10 ; 1e2..1e4 ]", this, null, "[ 0..XMAX ; 0..YMAX ; 10 ; 1e2..1e4 ]"));
			popup.addSeparator();
			popup.add(UI.createLabel(" Fonctions de distribution", UI.FONT_ITALIC));
			GraphCanvas cvDistrib;
			JMenu mnuDistribFunction;
			_Function func;
			for(int i=0; i<simulSpec.DISTRIBUTION_FUNCTIONS.size(); i++){
				func = (_Function)(simulSpec.DISTRIBUTION_FUNCTIONS.get(i));
				mnuDistribFunction = new JMenu(func.getDescription());
				cvDistrib = new GraphCanvas(-8, 4, 1, 0.2);
				cvDistrib.setYPolicy(-3, 10, 1, 0.2);
				cvDistrib.setScale(GraphCanvas.SCALE_BOTH_LOG);
				cvDistrib.setPreferredSize(new Dimension(350, 200));
				cvDistrib.clearFunctions();
				cvDistrib.addFunction(func);
				mnuDistribFunction.add(UI.createMenuItem("Ajouter", this, null, func.getName()));
				mnuDistribFunction.addSeparator();
				mnuDistribFunction.add(cvDistrib);
				popup.add(mnuDistribFunction);
			}
			popup.addSeparator();
			popup.add(UI.createMenuItem("Editeur de fonctions...", this, null));
			popup.addSeparator();
			popup.add(UI.createMenuItem("Annuler", null, null));
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "buildPopupMenu()", ex);}
	}
	

	

	
	
	
	
	
	


	public final void actionPerformed(ActionEvent ae)
	{
		super.actionPerformed(ae);
		try{
			if(ae.getSource() == btnAdd){
				popup.show(btnAdd, btnAdd.getWidth(), 0);
				return;
			}
			if(ae.getSource() == btnUpdate){
				/* recalcule les bruits */
				simulSpec.NOISES_FORMULA = txtFormula.getText();
				Generator.generate(simulSpec, Generator.GENERATE_FROM_NOISES);
			}
			if(ae.getSource() instanceof JMenuItem){
				txtFormula.setText(txtFormula.getText().substring(0, txtFormula.getSelectionStart()) + ((JMenuItem)(ae.getSource())).getActionCommand() + txtFormula.getText().substring(txtFormula.getSelectionEnd(), txtFormula.getText().length()));
				txtFormula.requestFocus();
			}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}
	
	public void onSimulationComponentsListChanged() {}
	public void onSimulationGeneratedComponentsChanged(int iGeneratedNumber) {}
	public void onSimulationStateChanged(boolean blSimulating) {btnUpdate.setEnabled(blSimulating?false:(simulSpec.getMatrix().isNormalized()?true:false));}

	public void onSimulationStartDistribution() {}
	public void onSimulationStartGeneration() {}
	public void onSimulationStartNoisation() {}
	public void onSimulationStartNormalization() {}

}
