/*
 SOS Project Classes.

 Copyright (C) 2006, 2007 SOS Project <http://sos.project.free.fr>
 SOS Project is :
 Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
 Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package ui.simulation.frames;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;

import kernel.Generator;
import kernel.Kernel;
import ressources.Lang;
import simulation.SimulationSpec;
import simulation._SimulationSpecListener;
import ui.UI;
import ui.common.DialogFrame;

public class NormalizationDialog extends DialogFrame implements ActionListener, _SimulationSpecListener
{

	private static final long serialVersionUID 			= 1L;

	private SimulationSpec simulSpec					= null;
	
	private JButton btnUpdate							= null;
	private JTextField txtNormaMono						= null;
	private JTextField txtNormaPuiss					= null;
	
	
	

	

	
	public NormalizationDialog(final SimulationSpec simulSpec)
	{
		super(simulSpec.getMainFrame(), Lang.STR_TITLE_NORMALIZATION, false);
		try{
			this.simulSpec = simulSpec;
			/* construit le contenu */
			GridBagConstraints c = new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 0, 3), 0, 0);
			JPanel pnlTmpText = UI.createPanel(new GridBagLayout());
			pnlTmpText.setBorder(new LineBorder(Color.darkGray, 1));
			/* delta de normalisation */
			pnlTmpText.add(UI.createLabel("  " + Lang.STR_OPT_IMAGE_NORM_DELTA, UI.FONT_ITALIC), c);
			c.gridx = 1;
			c.gridwidth = 2;
			pnlTmpText.add(txtNormaMono = new JTextField(""+simulSpec.NORMALISATION_VALUE), c);
			c.gridwidth = 1;
			/* puissance de normalisation */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("  " + Lang.STR_OPT_IMAGE_NORM_LOG, UI.FONT_ITALIC), c);
			c.gridx = 1;
			c.gridwidth = 2;
			pnlTmpText.add(txtNormaPuiss = new JTextField(""+simulSpec.NORMALISATION_POWER), c);
			c.gridwidth = 1;
			c.gridy++;
			/* mettre a jour */
			c.insets = new Insets(3, 3, 3, 3);
			c.gridx = 1;
			c.gridwidth = 2;
			pnlTmpText.add(btnUpdate = UI.createButton("Mettre à jour", this, null), c);
			btnUpdate.setEnabled(false);
			c.gridwidth = 1;
			/* ajoute le tout */
			JPanel pnl = UI.createPanel(new BorderLayout());
			pnl.add(UI.createLabel(Lang.STR_TITLE_NORMALIZATION, UI.FONT_BOLD_BIG), BorderLayout.NORTH);
			pnl.add(pnlTmpText, BorderLayout.CENTER);
			setContent(pnl, BUTTONS_CLOSE);
			setSize(340, this.getHeight());
			/* on demande l'ecoute de la simulation */
			this.simulSpec.addListener(this);
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "PanelNormalisation()", ex);}
	}


	
	
	

	
	
	private final void updateNormalisation()
	{
		try{
		
			/* on enregistre les valeurs */
			try{
				simulSpec.NORMALISATION_VALUE = Integer.parseInt(txtNormaMono.getText());
			}catch(Exception ex){
				txtNormaMono.setText(""+simulSpec.NORMALISATION_VALUE);
			}
			try{
				simulSpec.NORMALISATION_POWER = Double.parseDouble(txtNormaPuiss.getText());
			}catch(Exception ex){
				txtNormaPuiss.setText(""+simulSpec.NORMALISATION_POWER);
			}
			/* renormalise */
			Generator.generate(simulSpec, Generator.GENERATE_FROM_NORMALIZATION);
		
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "updateNormalisation()", ex);}
	}
	
	
	

	
	
	public final void actionPerformed(ActionEvent ae)
	{
		super.actionPerformed(ae);
		try{
			if(ae.getSource() == btnUpdate) {updateNormalisation(); return;}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}
	
	public void onSimulationComponentsListChanged() {}
	public void onSimulationGeneratedComponentsChanged(int iGeneratedNumber) {}
	public void onSimulationStateChanged(boolean blSimulating) {btnUpdate.setEnabled(blSimulating?false:(simulSpec.getMatrix().isNormalized()?true:false));}

	public void onSimulationStartDistribution() {}
	public void onSimulationStartGeneration() {}
	public void onSimulationStartNoisation() {}
	public void onSimulationStartNormalization() {}
	
	
	
}
