/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package ui.simulation.frames;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;

import kernel.Generator;
import kernel.Kernel;
import ressources.Lang;
import simulation.SimulationSpec;
import ui.UI;
import ui.common.DialogFrame;
import ui.simulation.managers.FiltersManager;

public class PropertiesDialog extends DialogFrame implements ActionListener
{

	private static final long serialVersionUID 			= 1L;

	private SimulationSpec simulSpec					= null;

	
	private JComboBox optMonoFilter						= null;
	private JButton btnEditFilters						= null;
	
	private JTextField txtSizeX							= null;
	private JTextField txtSizeY							= null;
	private JTextField txtPixelSize						= null;
	private JTextField txtObservationTime				= null;
	private JTextField txtTelescopeDiameter				= null;
	
	
	
	
	
	
	
	
	
	
	public PropertiesDialog(final SimulationSpec simulSpec)
	{
		super(simulSpec.getMainFrame(), Lang.STR_TITLE_PROPERTIES, true);
		try{
			this.simulSpec = simulSpec;
			/* construit le contenu */
			GridBagConstraints c = new GridBagConstraints();
			JPanel pnlTmpText = UI.createPanel(new GridBagLayout());
			pnlTmpText.setBorder(new LineBorder(Color.darkGray, 1));
			c.fill = GridBagConstraints.HORIZONTAL;
			c.gridwidth = 1;
			c.weightx = 0.1;
			c.weighty = 0.1;
			c.insets = new Insets(3, 3, 0, 3);
			/* image */
			c.gridx = 0;
			c.gridy = 0;
			pnlTmpText.add(UI.createLabel(Lang.STR_OPT_IMAGE_TITLE_ZONE, UI.FONT_BOLD), c);
			/* taille */
			c.gridy++;
			c.gridx = 0; 
			pnlTmpText.add(UI.createLabel("  " + Lang.STR_OPT_IMAGE_ZONE_SIZE, UI.FONT_ITALIC), c);
			c.gridx = 1;
			pnlTmpText.add(txtSizeX = new JTextField(""+simulSpec.PARAM_IMAGE_SIZE_X), c);
			c.gridx = 2;
			pnlTmpText.add(txtSizeY = new JTextField(""+simulSpec.PARAM_IMAGE_SIZE_Y), c);
			/* résolution */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("  " + Lang.STR_OPT_IMAGE_ZONE_RES, UI.FONT_ITALIC), c);
			c.gridx = 1; c.gridwidth = 2;
			pnlTmpText.add(txtPixelSize = new JTextField(""+simulSpec.PARAM_PIXEL_RESOLUTION), c);
			c.gridwidth = 1;
			/* aspects techniques */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("Aspects techniques", UI.FONT_BOLD), c);
			/* temps de pause */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("  " + "Temps de pause (sec)", UI.FONT_ITALIC), c);
			c.gridx = 1; c.gridwidth = 2;
			pnlTmpText.add(txtObservationTime = new JTextField(""+simulSpec.PARAM_OBSERVATION_TIME), c);
			c.gridwidth = 1;
			/* diametre du telescope */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("  " + "Diametre du telescope (m)", UI.FONT_ITALIC), c);
			c.gridx = 1; c.gridwidth = 2;
			pnlTmpText.add(txtTelescopeDiameter = new JTextField(""+simulSpec.PARAM_TELESCOPE_DIAMETER), c);
			c.gridwidth = 1;
			/* filtre */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("Filtre d'observation", UI.FONT_BOLD), c);
			/* selection */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("  " + Lang.STR_OPT_IMAGE_COLOR_FILTER_MONO, UI.FONT_ITALIC), c);
			c.gridx = 1; c.gridwidth = 2;
			optMonoFilter = new JComboBox();
			optMonoFilter.addActionListener(this);
			optMonoFilter.setModel(new DefaultComboBoxModel(simulSpec.getFilters()));
			optMonoFilter.setSelectedIndex(simulSpec.getFilterIndexMono());
			pnlTmpText.add(optMonoFilter, c);
			c.gridwidth = 1;
			/* editeur */
			c.insets = new Insets(3, 3, 3, 3);
			c.gridy++;
			c.gridx = 1; c.gridwidth = 2;
			pnlTmpText.add(btnEditFilters = UI.createButton("Editer les filtres", this, null), c);
			c.gridwidth = 1;
			/* ajoute le tout */
			JPanel pnl = UI.createPanel(new BorderLayout());
			pnl.add(UI.createLabel(Lang.STR_TITLE_PROPERTIES, UI.FONT_BOLD_BIG), BorderLayout.NORTH);
			pnl.add(pnlTmpText, BorderLayout.CENTER);
			setContent(pnl, BUTTONS_OK_CANCEL);
			setSize(500, this.getHeight());
			
			
			
			
	
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "PropertiesFrame()", ex);}
	}

	
	
	public void setVisible(boolean blVisible)
	{
		try{
			/* montre la fenetre */
			super.setVisible(blVisible);
			if((blVisible == true) && (this.retournValue() == DialogFrame.BUTTON_OK)){
				/* nouvelles valeurs */
				try{simulSpec.PARAM_IMAGE_SIZE_X = Integer.parseInt(txtSizeX.getText());}catch(Exception ex) {}
				try{simulSpec.PARAM_IMAGE_SIZE_Y = Integer.parseInt(txtSizeY.getText());}catch(Exception ex) {}
				try{simulSpec.PARAM_PIXEL_RESOLUTION = Double.parseDouble(txtPixelSize.getText());}catch(Exception ex) {}
				try{simulSpec.PARAM_OBSERVATION_TIME = Double.parseDouble(txtObservationTime.getText());}catch(Exception ex) {}
				try{simulSpec.PARAM_TELESCOPE_DIAMETER = Double.parseDouble(txtTelescopeDiameter.getText());}catch(Exception ex) {}
				try{simulSpec.setFilterIndexMono(optMonoFilter.getSelectedIndex());}catch(Exception ex) {}
				/* genere l'image a nouveau */
				Generator.generate(simulSpec, Generator.GENERATE_FROM_BEGINING);
			}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "setVisible()", ex);}
	}

	
	
	
	
	
	
	
	
	
	
	
	
	
	

	private final void showFilterEditor()
	{
		try{
			
			FiltersManager fm = new FiltersManager(simulSpec, this.getLocationOnScreen().x + 20, this.getLocationOnScreen().y + 20);
			fm.setVisible(true);
			if(fm.retournValue() == DialogFrame.BUTTON_OK){
				optMonoFilter.removeAllItems();
				optMonoFilter.setModel(new DefaultComboBoxModel(simulSpec.getFilters().toArray()));
			}
			
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "toggleMonoFilter()", ex);}
	}

	public final void actionPerformed(ActionEvent ae)
	{
		super.actionPerformed(ae);
		try{
			if(ae.getSource() == btnEditFilters) {showFilterEditor(); return;}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}


}
