/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package ui.simulation.managers;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;

import kernel.Kernel;
import kernel.functions.FunctionsFactory;
import kernel.functions._Function;
import ressources.Lang;
import simulation.SimulationSpec;
import ui.UI;
import ui.common.DialogFrame;

public class DistribFuncsManager extends DialogFrame implements ActionListener
{

	private static final long serialVersionUID 			= 1L;

	private SimulationSpec simulSpec					= null;
	
	private Vector data									= new Vector();

	private JComboBox optFuncs							= null;
	
	private JTextField txtFunctionName					= null;
	private JTextArea txtFunction						= null;
	
	private JButton btnAddFunc							= null;
	private JButton btnDelFunc							= null;
	



	
	public DistribFuncsManager(final SimulationSpec simulSpec, final int x, final int y)
	{
		super(simulSpec.getMainFrame(), Lang.STR_TITLE_DISTRIB_FUNCS_MANAGER, true, x, y);
		try{
			this.simulSpec = simulSpec;
			/* construit le contenu */
			GridBagConstraints c = new GridBagConstraints();
			JPanel pnlTmpText = UI.createPanel(new GridBagLayout());
			pnlTmpText.setBorder(new LineBorder(Color.darkGray, 1));
			
			
			for(int i=0; i<simulSpec.DISTRIBUTION_FUNCTIONS.size(); i++) data.add(simulSpec.DISTRIBUTION_FUNCTIONS.get(i));
	
			c.fill = GridBagConstraints.BOTH;
			c.weightx = 0.1;
			c.weighty = 0.1;
			c.insets = new Insets(3, 3, 0, 3);
			
			/* liste des fonctions */
			c.gridx = 0;
			c.gridy = 0;
			pnlTmpText.add(UI.createLabel("  " + "Liste des fonctions", UI.FONT_ITALIC), c);
			c.gridx = 1;
			c.gridwidth = 2;
			pnlTmpText.add(optFuncs = UI.createComboBox(data.toArray(), this, simulSpec.getFilterIndexMono()), c);
			c.gridwidth = 1;
			
			/* description */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("  " + "Description", UI.FONT_ITALIC), c);
			c.gridx = 1; c.gridwidth = 2;
			pnlTmpText.add(txtFunctionName = new JTextField(""), c);
			c.gridwidth = 1;
			
			/* fonctions */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("  " + "Fonction", UI.FONT_ITALIC), c);
			c.gridx = 1; c.gridwidth = 2;
			pnlTmpText.add(UI.createScrollPane(txtFunction = UI.createTextArea(""), new Dimension(300, 100)), c);
			c.gridwidth = 1;
			
			
			
			
			/* boutons */
			c.gridy++;
			c.gridx = 0;
			c.insets = new Insets(3, 3, 3, 3);
			pnlTmpText.add(UI.createLabel("  " + "Actions", UI.FONT_ITALIC), c);
			c.gridx = 1;
			pnlTmpText.add(btnDelFunc = UI.createButton("Supprimer", this, null), c);
			c.gridx = 2;
			pnlTmpText.add(btnAddFunc = UI.createButton("Ajouter", this, null), c);
			
			toggleFunc();
			
			
			
			
			/* ajoute le tout */
			JPanel pnl = UI.createPanel(new BorderLayout());
			pnl.add(UI.createLabel(Lang.STR_TITLE_DISTRIB_FUNCS_MANAGER, UI.FONT_BOLD_BIG), BorderLayout.NORTH);
			pnl.add(pnlTmpText, BorderLayout.CENTER);
			setContent(pnl, BUTTONS_OK_CANCEL);
			setSize(500, this.getHeight());

		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "DistribFuncsManager()", ex);}
	}

	
	public void setVisible(boolean blVisible)
	{
		try{
			/* montre la fenetre */
			super.setVisible(blVisible);
			if((blVisible == true) && (this.retournValue() == DialogFrame.BUTTON_OK)){
				/* nouvelles valeurs */
				if(this.retournValue() == DialogFrame.BUTTON_OK){
					simulSpec.DISTRIBUTION_FUNCTIONS = data;
					//simulSpec.setLastDistribFunctionIndex(0);
				}
			}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "setVisible()", ex);}
	}

	
	
	
	
	
	
	
	
	
	
	
	
	
	




	
	
	private final void updateFuncs()
	{
		try{
			optFuncs.removeAllItems();
			optFuncs.setModel(new DefaultComboBoxModel(data.toArray()));
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "updateFilters()", ex);}
	}

	private final void addFunc()
	{
		try{
			data.add(FunctionsFactory.parse(txtFunctionName.getText(), txtFunctionName.getText(), txtFunction.getText(), null));
			updateFuncs();
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "addFilter()", ex);}
	}

	private final void delFunc()
	{
		try{
			if(optFuncs.getSelectedIndex() > -1){
				data.remove(optFuncs.getSelectedIndex());
				updateFuncs();
			}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "delFilter()", ex);}
	}

	private final void toggleFunc()
	{
		try{
			if(optFuncs.getSelectedIndex() > -1){
				_Function f = (_Function)(data.get(optFuncs.getSelectedIndex()));
				txtFunctionName.setText(f.getDescription());
				txtFunction.setText(f.getFunction());
			}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "toggleMonoFilter()", ex);}
	}
	
	public final void actionPerformed(ActionEvent ae)
	{
		super.actionPerformed(ae);
		try{
			
			if(ae.getSource() == optFuncs) {toggleFunc(); return;}
			
			if(ae.getSource() == btnDelFunc) {delFunc(); return;}
			
			if(ae.getSource() == btnAddFunc) {addFunc(); return;}

		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}

}
