/*
  SOS Project Classes.

  Copyright (C) 2006 SOS Project <http://sos.project.free.fr>
  SOS Project is :
    Denis BURGARELLA @ OAMP <http://www.lam.oamp.fr/denis>
    Camille AMBROSI @ CMI <http://camille.ambrosi.free.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package ui.simulation.managers;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;

import kernel.Kernel;
import kernel.Maths;
import ressources.Lang;
import simulation.SimulationSpec;
import simulation.components.Filter;
import ui.UI;
import ui.common.DialogFrame;

public class FiltersManager extends DialogFrame implements ActionListener
{
	

	private static final long serialVersionUID 			= 1L;
	
	private SimulationSpec simulSpec					= null;

	private Vector data									= new Vector();
	
	private JComboBox optFilters						= null;
	
	private JTextField txtFilterName					= null;
	private JTextField txtFilterCenter					= null;
	private JTextField txtFilterDelta					= null;
	private JTextField txtFilterR						= null;
	
	private JButton btnAddFilter						= null;
	private JButton btnDelFilter						= null;
	



	
	public FiltersManager(final SimulationSpec simulSpec, final int x, final int y)
	{
		super(simulSpec.getMainFrame(), Lang.STR_TITLE_FILTERS_MANAGER, true, x, y);
		try{
			this.simulSpec = simulSpec;
			/* construit le contenu */
			GridBagConstraints c = new GridBagConstraints();
			JPanel pnlTmpText = UI.createPanel(new GridBagLayout());
			pnlTmpText.setBorder(new LineBorder(Color.darkGray, 1));


			
			for(int i=0; i<simulSpec.getFilters().size(); i++) data.add(simulSpec.getFilters().get(i));
	
			c.fill = GridBagConstraints.BOTH;
			c.weightx = 0.1;
			c.weighty = 0.1;
			c.insets = new Insets(3, 3, 0, 3);
			
			/* liste des filtres */
			c.gridx = 0;
			c.gridy = 0;
			pnlTmpText.add(UI.createLabel("  " + "Liste des filtres", UI.FONT_ITALIC), c);
			c.gridx = 1; c.gridwidth = 2;
			pnlTmpText.add(optFilters = UI.createComboBox(data.toArray(), this, simulSpec.getFilterIndexMono()), c);
			c.gridwidth = 1;
			
			/* nom du filtre */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("  " + "Description", UI.FONT_ITALIC), c);
			c.gridx = 1; c.gridwidth = 2;
			pnlTmpText.add(txtFilterName = new JTextField(""), c);
			c.gridwidth = 1;
			
			/* valeurs */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("  " + "Valeur (centre et largeur) (m)", UI.FONT_ITALIC), c);
			c.gridx = 1;
			pnlTmpText.add(txtFilterCenter = new JTextField(""), c);
			c.gridx = 2;
			pnlTmpText.add(txtFilterDelta = new JTextField(""), c);
			
			/* valeurs */
			c.gridy++;
			c.gridx = 0;
			pnlTmpText.add(UI.createLabel("  " + "R (lambda/dlambda) (m)", UI.FONT_ITALIC), c);
			c.gridx = 2;
			pnlTmpText.add(txtFilterR = new JTextField(""), c);
			txtFilterR.setEditable(false);
			
	
			/* boutons */
			c.gridy++;
			c.gridx = 0;
			c.insets = new Insets(3, 3, 3, 3);
			pnlTmpText.add(UI.createLabel("  " + "Actions", UI.FONT_ITALIC), c);
			c.gridx = 1;
			pnlTmpText.add(btnDelFilter = UI.createButton("Supprimer", this, null), c);
			c.gridx = 2;
			pnlTmpText.add(btnAddFilter = UI.createButton("Ajouter", this, null), c);
			
			toggleFilter();
			
			
			/* ajoute le tout */
			JPanel pnl = UI.createPanel(new BorderLayout());
			pnl.add(UI.createLabel(Lang.STR_TITLE_FILTERS_MANAGER, UI.FONT_BOLD_BIG), BorderLayout.NORTH);
			pnl.add(pnlTmpText, BorderLayout.CENTER);
			setContent(pnl, BUTTONS_OK_CANCEL);
			setSize(500, this.getHeight());
			
			
			

		
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "FiltersManager()", ex);}
	}
	

	
	public void setVisible(boolean blVisible)
	{
		try{
			/* montre la fenetre */
			super.setVisible(blVisible);
			if((blVisible == true) && (this.retournValue() == DialogFrame.BUTTON_OK)){
				/* nouvelles valeurs */
				if(this.retournValue() == DialogFrame.BUTTON_OK){
					simulSpec.setFilters(data);
					simulSpec.setFilterIndexMono(0);
				}
			}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "setVisible()", ex);}
	}

	
	
	
	
	
	




	
	
	private final void updateFilters()
	{
		try{
			optFilters.removeAllItems();
			optFilters.setModel(new DefaultComboBoxModel(data.toArray()));
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "updateFilters()", ex);}
	}

	private final void addFilter()
	{
		try{
			data.add(new Filter(txtFilterName.getText(), Double.parseDouble(txtFilterCenter.getText()), Double.parseDouble(txtFilterDelta.getText())));
			updateFilters();
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "addFilter()", ex);}
	}

	private final void delFilter()
	{
		try{
			if(optFilters.getSelectedIndex() > -1){
				data.remove(optFilters.getSelectedIndex());
				updateFilters();
			}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "delFilter()", ex);}
	}

	private final void toggleFilter()
	{
		try{
			if(optFilters.getSelectedIndex() > -1){
				Filter f = (Filter)(data.get(optFilters.getSelectedIndex()));
				txtFilterName.setText(f.getDescription());
				txtFilterCenter.setText(Maths.toString(f.lambdaCenterInMeter(), -1));
				txtFilterDelta.setText(Maths.toString(f.dlambdaInMeter(), -1));
				txtFilterR.setText(Maths.toString( f.lambdaCenterInMeter() / f.dlambdaInMeter() , 2));
			}
		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "toggleMonoFilter()", ex);}
	}
	
	public final void actionPerformed(ActionEvent ae)
	{
		super.actionPerformed(ae);
		try{
			
			if(ae.getSource() == optFilters) {toggleFilter(); return;}
			
			if(ae.getSource() == btnDelFilter) {delFilter(); return;}
			
			if(ae.getSource() == btnAddFilter) {addFilter(); return;}

		}catch(Exception ex) {Kernel.fireError(this.getClass().getName(), "actionPerformed()", ex);}
	}

}
